/*******************************************************************************
 * FileName:        echo.h
 * Processor:       PIC32
 * Compiler:        CX32
 * Company:         Pacific Coast Controls
 * Created:         August 20, 2015, 11:39 AM
 ******************************************************************************/
#ifndef ECHO_H
#define	ECHO_H

#ifdef	__cplusplus
extern "C" {
#endif

//Standard includes.
#include <p32xxxx.h>
#include <plib.h>
#include <stdio.h>

//Kernel includes.
#include "FreeRTOS/include/FreeRTOS.h"
#include "FreeRTOS/include/list.h"
#include "FreeRTOS/include/queue.h"
#include "FreeRTOS/include/semphr.h"
#include "FreeRTOS/include/task.h"

/****Labels********************************************************************/
//System
#define SYS_FREQ (80000000L)
#define	GetPeripheralClock()		(SYS_FREQ/(1 << OSCCONbits.PBDIV))

//Comport
#define comBUFFER_LEN                   50
#define mainCOM_TEST_BAUD_RATE		(115200)

//task priorities
#define mainLED_TASK_PRIORITY		(tskIDLE_PRIORITY + 1)
#define mainButton_TASK_PRIORITY        (tskIDLE_PRIORITY + 2)
#define mainUSART_TASK_PRIORITY         (tskIDLE_PRIORITY)

//Button controls
#define SW0_TRIS()                      mPORTGSetPinsDigitalIn(BIT_15)
#define	SW0_IO                          mPORTGReadBits(BIT_15)
#define DEBOUNCECOUNTS                  5

//LED controls
#define LEDtris()                       mPORTFSetPinsDigitalOut(0x000B)
#define LEDclr()                        mPORTFClearBits(0x000B)

#define LED1on()                        (LATFSET = 0x08)
#define LED2on()                        (LATFSET = 0x01)
#define LED3on()                        (LATFSET = 0x02)

#define LED1inv()                       (LATFINV = 0x08)
#define LED2inv()                       (LATFINV = 0x01)
#define LED3inv()                       (LATFINV = 0x02)

#define LED1off()                       (LATFCLR = 0x08)
#define LED2off()                       (LATFCLR = 0x01)
#define LED3off()                       (LATFCLR = 0x02)

/****Global Variables**********************************************************/
//Task handle for LED 3
TaskHandle_t LED3_Hndl;

typedef struct
{
    UINT8 LED;
    portTickType Period;
}Params;

/****Prototypes****************************************************************/
void SetupHardware( unsigned long ulWantedBaud, unsigned portBASE_TYPE uxQueueLength );
void vLEDFlashTask( void *pvParameters );
void vButtonLEDTask( void *pvParameters );
void vButtonCheckTask( void *pvParameters );
void RxQueueToTxQueue( void *pvParameters );
void SendStringUart1( void *pvParameters );


#ifdef	__cplusplus
}
#endif

#endif	/* ECHO_H */

