/*
 * cli_uart.c - msp430fr5969 launchpad application uart interface implementation
 *
 * Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/
#include <string.h>
#include <plib.h>
#include "CC3100/cli_uart.h"

#define ASCII_ENTER     0x0D

/*******************************************************************************
 * brief Read the data on Application UArt channel, User is expected to assign
 *       memory and pass the pointer to buffer to the function
 * param[in]  pBuff - pointer to the read buffer User should allocate the memory
 *            to the buffer and pass the pointer to API
 * return int - No of bytes read, -1 in case of error
 ******************************************************************************/
INT16 CLI_Read(unsigned char *pBuff)
{
    if(pBuff == NULL)
        return -1;

    while(1)
    {
        while(!UARTReceivedDataIsAvailable(UART1));
        *pBuff = UARTGetDataByte(UART1);

        if(*pBuff == ASCII_ENTER)break;
        pBuff++;
    }
    
    return strlen((const char *)pBuff);
}

/*******************************************************************************
 * brief Write the data on Application Uart channel
 * param[in] inBuff - Pointer to output data, string should be null terminated
 * return     int - No of byte sent, -1 in case of error
 ******************************************************************************/
INT16 CLI_Write(unsigned char *inBuff)
{
    unsigned short ret,usLength;

    if(inBuff == NULL)
        return -1;

    usLength = strlen((const char *)inBuff);
    ret = usLength;
    
    while (usLength)
    {
        while(!UARTTransmitterIsReady(UART1));
        UARTSendDataByte(UART1, *inBuff);

        usLength--;
        inBuff++;
    }
    return (int)ret;

}

/*******************************************************************************
 * brief Configures the Application Uart
 * param[in] none
 * return none
 ******************************************************************************/
void CLI_Configure(UINT32 sourceClock, UINT32 dataRate)
{
    UART_CONFIGURATION flags;
    
    if(dataRate >= 57600L)
        {
        flags = UART_ENABLE_PINS_TX_RX_ONLY|UART_ENABLE_HIGH_SPEED;
        }
    else
        {
        flags = UART_ENABLE_PINS_TX_RX_ONLY;
        }

    UARTConfigure(UART1, flags);
    UARTSetFifoMode(UART1, UART_INTERRUPT_ON_TX_NOT_FULL | UART_INTERRUPT_ON_RX_NOT_EMPTY);
    UARTSetLineControl(UART1, UART_DATA_SIZE_8_BITS | UART_PARITY_NONE | UART_STOP_BITS_1);
    UARTSetDataRate(UART1, sourceClock, dataRate);
    UARTEnable(UART1, UART_ENABLE_FLAGS(UART_PERIPHERAL | UART_RX | UART_TX));

}


