/*
 * device.h - CC31xx/CC32xx Host Driver Implementation
 *
 * Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/
#include "CC3100/include/simplelink.h"

#ifndef __DEVICE_H__
#define __DEVICE_H__

#ifdef __cplusplus
extern "C" {
#endif

/*******************************************************************************
 * SL internal Error codes
 ******************************************************************************/
       
/* Receive this error in case there are no resources to issue the command
   If possible, increase the number of MAX_CUNCURENT_ACTIONS (result in memory increase)
   If not, try again later */
#define SL_POOL_IS_EMPTY (-2000)
         
/* Receive this error in case a given length for RX buffer was too small. 
   Receive payload was bigger than the given buffer size. Therefore, payload is cut according to receive size 
   Recommend to increase buffer size */
#define SL_ESMALLBUF     (-2001)

/* Receive this error in case zero length is supplied to a "get" API
   Recommend to supply length according to requested information (view options defines for help) */
#define SL_EZEROLEN      (-2002)

/* User supplied invalid parameter */
#define SL_INVALPARAM    (-2003)

/* Failed to open interface  */
#define SL_BAD_INTERFACE    (-2004)

/*******************************************************************************
 * Errors returned from the general error async event
 ******************************************************************************/
/* Send types */
typedef enum
{
    SL_ERR_SENDER_HEALTH_MON,
    SL_ERR_SENDER_CLI_UART,
    SL_ERR_SENDER_SUPPLICANT,
    SL_ERR_SENDER_NETWORK_STACK,
    SL_ERR_SENDER_WLAN_DRV_IF,
    SL_ERR_SENDER_WILINK,
    SL_ERR_SENDER_INIT_APP,
    SL_ERR_SENDER_NETX,
    SL_ERR_SENDER_HOST_APD,
    SL_ERR_SENDER_MDNS,
    SL_ERR_SENDER_HTTP_SERVER,
    SL_ERR_SENDER_DHCP_SERVER,
    SL_ERR_SENDER_DHCP_CLIENT,
    SL_ERR_DISPATCHER,
    SL_ERR_NUM_SENDER_LAST=0xFF
}SlErrorSender_e; 

/* Error codes */
#define SL_ERROR_STATIC_ADDR_SUBNET_ERROR                   (-60)  /* network stack error*/
#define SL_ERROR_ILLEGAL_CHANNEL                            (-61)  /* supplicant error */
#define SL_ERROR_SUPPLICANT_ERROR                           (-72)  /* init error code */
#define SL_ERROR_HOSTAPD_INIT_FAIL                          (-73)  /* init error code */
#define SL_ERROR_HOSTAPD_INIT_IF_FAIL                       (-74)  /* init error code */
#define SL_ERROR_WLAN_DRV_INIT_FAIL                         (-75)  /* init error code */
#define SL_ERROR_WLAN_DRV_START_FAIL                        (-76)  /* wlan start error */
#define SL_ERROR_FS_FILE_TABLE_LOAD_FAILED                  (-77)  /* init file system failed */
#define SL_ERROR_PREFERRED_NETWORKS_FILE_LOAD_FAILED        (-78)  /* init file system failed */
#define SL_ERROR_HOSTAPD_BSSID_VALIDATION_ERROR             (-79)  /* Ap configurations BSSID error */
#define SL_ERROR_HOSTAPD_FAILED_TO_SETUP_INTERFACE          (-80)  /* Ap configurations interface error */
#define SL_ERROR_MDNS_ENABLE_FAIL                           (-81)  /* mDNS enable failed                */
#define SL_ERROR_HTTP_SERVER_ENABLE_FAILED                  (-82)  /* HTTP server enable failed         */
#define SL_ERROR_DHCP_SERVER_ENABLE_FAILED                  (-83)  /* DHCP server enable failed         */
#define SL_ERROR_PREFERRED_NETWORK_LIST_FULL                (-93)  /* supplicant error  */
#define SL_ERROR_PREFERRED_NETWORKS_FILE_WRITE_FAILED       (-94)  /* supplicant error  */
#define SL_ERROR_DHCP_CLIENT_RENEW_FAILED                   (-100) /* DHCP client error */
/* WLAN Connection management status */
#define SL_ERROR_CON_MGMT_STATUS_UNSPECIFIED                (-102)  
#define SL_ERROR_CON_MGMT_STATUS_AUTH_REJECT                (-103)  
#define SL_ERROR_CON_MGMT_STATUS_ASSOC_REJECT               (-104)  
#define SL_ERROR_CON_MGMT_STATUS_SECURITY_FAILURE           (-105)  
#define SL_ERROR_CON_MGMT_STATUS_AP_DEAUTHENTICATE          (-106)  
#define SL_ERROR_CON_MGMT_STATUS_AP_DISASSOCIATE            (-107)  
#define SL_ERROR_CON_MGMT_STATUS_ROAMING_TRIGGER            (-108)  
#define SL_ERROR_CON_MGMT_STATUS_DISCONNECT_DURING_CONNECT  (-109)  
#define SL_ERROR_CON_MGMT_STATUS_SG_RESELECT                (-110)  
#define SL_ERROR_CON_MGMT_STATUS_ROC_FAILURE                (-111)  
#define SL_ERROR_CON_MGMT_STATUS_MIC_FAILURE                (-112)  
/* end of WLAN connection management error statuses */
#define SL_ERROR_WAKELOCK_ERROR_PREFIX                      (-115)  /* Wake lock expired */
#define SL_ERROR_LENGTH_ERROR_PREFIX                        (-116)  /* Uart header length error */
#define SL_ERROR_MDNS_CREATE_FAIL                           (-121)  /* mDNS create failed */
#define SL_ERROR_GENERAL_ERROR                              (-127)

#define SL_DEVICE_GENERAL_CONFIGURATION           (1)
#define SL_DEVICE_GENERAL_CONFIGURATION_DATE_TIME (11)
#define SL_DEVICE_GENERAL_VERSION                 (12)
#define SL_DEVICE_STATUS                          (2)

/* SL_EVENT_CLASS_WLAN connection user events */
#define SL_WLAN_CONNECT_EVENT                     (1)
#define SL_WLAN_DISCONNECT_EVENT                  (2)
/* WLAN Smart Config user events */
#define SL_WLAN_SMART_CONFIG_COMPLETE_EVENT       (3)
#define SL_WLAN_SMART_CONFIG_STOP_EVENT           (4)
/* WLAN AP user events */
#define SL_WLAN_STA_CONNECTED_EVENT               (5)
#define SL_WLAN_STA_DISCONNECTED_EVENT            (6)
/* WLAN P2P user events */
#define SL_WLAN_P2P_DEV_FOUND_EVENT               (7)
#define SL_WLAN_P2P_NEG_REQ_RECEIVED_EVENT     	  (8)
#define SL_WLAN_CONNECTION_FAILED_EVENT           (9)
/* SL_EVENT_CLASS_DEVICE user events */
#define SL_DEVICE_FATAL_ERROR_EVENT               (1)
#define SL_DEVICE_ABORT_ERROR_EVENT               (2)

/* SL_EVENT_CLASS_BSD user events */               
#define    SL_SOCKET_TX_FAILED_EVENT              (1) 
#define SL_SOCKET_ASYNC_EVENT                     (2)
/* SL_EVENT_CLASS_NETAPP user events */   
#define    SL_NETAPP_IPV4_IPACQUIRED_EVENT        (1)
#define    SL_NETAPP_IPV6_IPACQUIRED_EVENT        (2)
#define SL_NETAPP_IP_LEASED_EVENT                 (3)
#define SL_NETAPP_IP_RELEASED_EVENT               (4)

/* Server Events */
#define SL_NETAPP_HTTPGETTOKENVALUE_EVENT         (1)
#define SL_NETAPP_HTTPPOSTTOKENVALUE_EVENT        (2)

/* Events list to mask/unmask*/
#define SL_EVENT_CLASS_GLOBAL                     (0)
#define SL_EVENT_CLASS_DEVICE                     (1)
#define SL_EVENT_CLASS_WLAN                       (2)
#define SL_EVENT_CLASS_BSD                        (3)
#define SL_EVENT_CLASS_NETAPP                     (4)
#define SL_EVENT_CLASS_NETCFG                     (5)
#define SL_EVENT_CLASS_FS                         (6)

/******************  DEVICE CLASS status ****************/
#define EVENT_DROPPED_DEVICE_ASYNC_GENERAL_ERROR          (0x00000001L)
#define STATUS_DEVICE_SMART_CONFIG_ACTIVE                 (0x80000000L)
  
/******************  WLAN CLASS status ****************/
#define EVENT_DROPPED_WLAN_WLANASYNCONNECTEDRESPONSE      (0x00000001L)
#define EVENT_DROPPED_WLAN_WLANASYNCDISCONNECTEDRESPONSE  (0x00000002L)
#define EVENT_DROPPED_WLAN_STA_CONNECTED                  (0x00000004L)
#define EVENT_DROPPED_WLAN_STA_DISCONNECTED               (0x00000008L)
#define STATUS_WLAN_STA_CONNECTED                         (0x80000000L)
                      
/******************  NETAPP CLASS status ****************/
#define EVENT_DROPPED_NETAPP_IPACQUIRED                   (0x00000001L)
#define EVENT_DROPPED_NETAPP_IPACQUIRED_V6                (0x00000002L)
#define EVENT_DROPPED_NETAPP_IP_LEASED                    (0x00000004L)
#define EVENT_DROPPED_NETAPP_IP_RELEASED                  (0x00000008L)
                      
/******************  BSD CLASS status *******************/
#define EVENT_DROPPED_SOCKET_TXFAILEDASYNCRESPONSE        (0x00000001L)
  
/******************  FS CLASS  **************************/
  
/*******************************************************************************
 * Structure/Enum declarations
 ******************************************************************************/
#define ROLE_UNKNOWN_ERR   (-1)

typedef struct
{
    UINT32 ChipId;
    UINT32 FwVersion[4];
    UINT8  PhyVersion[4];
}_SlPartialVersion;

typedef struct
{
    _SlPartialVersion ChipFwAndPhyVersion;
    UINT32            NwpVersion[4];
    UINT16            RomVersion;
    UINT16            Padding;
}SlVersionFull;

typedef struct
{
    UINT32 AbortType;
    UINT32 AbortData;
}sl_DeviceReportAbort;

typedef struct
{
    INT8            status;
    SlErrorSender_e sender;
}sl_DeviceReport;

typedef union
{
  sl_DeviceReport       deviceEvent; 
  sl_DeviceReportAbort	deviceReport;  
} _SlDeviceEventData_u;

typedef struct
{
   UINT32               Event;
   _SlDeviceEventData_u EventData;
} SlDeviceEvent_t;

typedef struct  
{
       /* time */
    UINT32 sl_tm_sec;
    UINT32 sl_tm_min;
    UINT32 sl_tm_hour;
       /* date */
    UINT32 sl_tm_day; /* 1-31 */
    UINT32 sl_tm_mon; /* 1-12 */
    UINT32 sl_tm_year; /*  YYYY 4 digits  */
    UINT32 sl_tm_week_day; /* not required */
    UINT32 sl_tm_year_day; /* not required */
    UINT32 reserved[3];
}SlDateTime_t;

/*******************************************************************************
 * Type declarations
 *******************************************************************************/
typedef void (*P_INIT_CALLBACK)(_u32 Status);

/*******************************************************************************
 * Function prototypes
 ******************************************************************************/
#if _SL_INCLUDE_FUNC(sl_Start)
INT16 sl_Start(INT8*  pDevName, const P_INIT_CALLBACK pInitCallBack);
#endif

#if _SL_INCLUDE_FUNC(sl_Stop)
INT16 sl_Stop(const UINT16 timeout);
#endif

#if _SL_INCLUDE_FUNC(sl_DevSet)
INT32 sl_DevSet(const UINT8 DeviceSetId ,const UINT8 Option,const UINT8 ConfigLen,const UINT8 *pValues);
#endif

#if _SL_INCLUDE_FUNC(sl_DevGet)
INT32 sl_DevGet(const UINT8 DeviceGetId,UINT8 *pOption,UINT8 *pConfigLen, UINT8 *pValues);
#endif

#if _SL_INCLUDE_FUNC(sl_EventMaskSet)
INT16 sl_EventMaskSet(const UINT8 EventClass ,const UINT32 Mask);
#endif

#if _SL_INCLUDE_FUNC(sl_EventMaskGet)
INT16 sl_EventMaskGet(const UINT8 EventClass,UINT32 *pMask);
#endif

#if _SL_INCLUDE_FUNC(sl_Task)
void sl_Task(void);
#endif

#ifdef  __cplusplus
}
#endif /*  __cplusplus */

#endif  /*  __DEVICE_H__ */


