/*
 * netapp.h - CC31xx/CC32xx Host Driver Implementation
 *
 * Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/
#include "CC3100/include/simplelink.h"

#ifndef __NETAPP_H__
#define __NETAPP_H__

#ifdef  __cplusplus
extern "C" {
#endif

/*****************************************************************************/
/* Macro declarations                                                        */
/*****************************************************************************/
/*ERROR code*/
#define SL_ERROR_NETAPP_RX_BUFFER_LENGTH_ERROR      (-230)

/*  Http Server interface */
#define MAX_INPUT_STRING                            (64) /*  because of WPA */

#define MAX_AUTH_NAME_LEN                           (20)
#define MAX_AUTH_PASSWORD_LEN                       (20)
#define MAX_AUTH_REALM_LEN                          (20)

#define MAX_DEVICE_URN_LEN                          (15+1)
#define MAX_DOMAIN_NAME_LEN                         (24+1)

#define MAX_ACTION_LEN                              (30)
/* Important: in case the max len is changed, make sure the struct sl_NetAppHttpServerSendToken_t in protocol.h is padded correctly! */
#define MAX_TOKEN_NAME_LEN                          (20)
#define MAX_TOKEN_VALUE_LEN                         MAX_INPUT_STRING

#define NETAPP_MAX_SERVICE_TEXT_SIZE                  (256)
#define NETAPP_MAX_SERVICE_NAME_SIZE                  (60)
#define NETAPP_MAX_SERVICE_HOST_NAME_SIZE             (64)

/* Server Responses */
#define SL_NETAPP_RESPONSE_NONE                       (0)
#define SL_NETAPP_HTTPSETTOKENVALUE                   (1)

#define SL_NETAPP_FAMILY_MASK                         (0x80)

/* mDNS types */
#define SL_NET_APP_MASK_IPP_TYPE_OF_SERVICE           (0x00000001)
#define SL_NET_APP_MASK_DEVICE_INFO_TYPE_OF_SERVICE   (0x00000002)
#define SL_NET_APP_MASK_HTTP_TYPE_OF_SERVICE          (0x00000004)
#define SL_NET_APP_MASK_HTTPS_TYPE_OF_SERVICE         (0x00000008)
#define SL_NET_APP_MASK_WORKSATION_TYPE_OF_SERVICE    (0x00000010)
#define SL_NET_APP_MASK_GUID_TYPE_OF_SERVICE          (0x00000020)
#define SL_NET_APP_MASK_H323_TYPE_OF_SERVICE          (0x00000040)
#define SL_NET_APP_MASK_NTP_TYPE_OF_SERVICE           (0x00000080)
#define SL_NET_APP_MASK_OBJECITVE_TYPE_OF_SERVICE     (0x00000100)
#define SL_NET_APP_MASK_RDP_TYPE_OF_SERVICE           (0x00000200)
#define SL_NET_APP_MASK_REMOTE_TYPE_OF_SERVICE        (0x00000400)
#define SL_NET_APP_MASK_RTSP_TYPE_OF_SERVICE          (0x00000800)
#define SL_NET_APP_MASK_SIP_TYPE_OF_SERVICE           (0x00001000)
#define SL_NET_APP_MASK_SMB_TYPE_OF_SERVICE           (0x00002000)
#define SL_NET_APP_MASK_SOAP_TYPE_OF_SERVICE          (0x00004000)
#define SL_NET_APP_MASK_SSH_TYPE_OF_SERVICE           (0x00008000)
#define SL_NET_APP_MASK_TELNET_TYPE_OF_SERVICE        (0x00010000)
#define SL_NET_APP_MASK_TFTP_TYPE_OF_SERVICE          (0x00020000)
#define SL_NET_APP_MASK_XMPP_CLIENT_TYPE_OF_SERVICE   (0x00040000)
#define SL_NET_APP_MASK_RAOP_TYPE_OF_SERVICE          (0x00080000)
#define SL_NET_APP_MASK_ALL_TYPE_OF_SERVICE           (0xFFFFFFFF)

/********************************************************************************************************/
/* sl_NetAppDnsGetHostByName error codes     */

#define SL_NET_APP_DNS_QUERY_NO_RESPONSE              (-159)  /* DNS query failed, no response                        */ 
#define SL_NET_APP_DNS_NO_SERVER                      (-161)  /* No DNS server was specified                          */ 
#define SL_NET_APP_DNS_PARAM_ERROR                    (-162)  /* mDNS parameters error                                */
#define SL_NET_APP_DNS_QUERY_FAILED                   (-163)  /* DNS query failed; no DNS server sent an 'answer'     */ 
#define SL_NET_APP_DNS_INTERNAL_1                     (-164)
#define SL_NET_APP_DNS_INTERNAL_2                     (-165)
#define SL_NET_APP_DNS_MALFORMED_PACKET               (-166)  /* Improperly formed or corrupted DNS packet received   */ 
#define SL_NET_APP_DNS_INTERNAL_3                     (-167)
#define SL_NET_APP_DNS_INTERNAL_4                     (-168)
#define SL_NET_APP_DNS_INTERNAL_5                     (-169)
#define SL_NET_APP_DNS_INTERNAL_6                     (-170)
#define SL_NET_APP_DNS_INTERNAL_7                     (-171)
#define SL_NET_APP_DNS_INTERNAL_8                     (-172)
#define SL_NET_APP_DNS_INTERNAL_9                     (-173)
#define SL_NET_APP_DNS_MISMATCHED_RESPONSE            (-174)  /* Server response type does not match the query request*/
#define SL_NET_APP_DNS_INTERNAL_10                    (-175)
#define SL_NET_APP_DNS_INTERNAL_11                    (-176)
#define SL_NET_APP_DNS_NO_ANSWER                      (-177)  /* No response for one-shot query */
#define SL_NET_APP_DNS_NO_KNOWN_ANSWER                (-178)  /* No known answer for query */
#define SL_NET_APP_DNS_NAME_MISMATCH                  (-179)  /* Illegal service name according to the RFC            */
#define SL_NET_APP_DNS_NOT_STARTED                    (-180)  /* mDNS is not running                                  */
#define SL_NET_APP_DNS_HOST_NAME_ERROR                (-181)  /* Host name error. Host name format is not allowed according to RFC 1033,1034,1035, 6763 */
#define SL_NET_APP_DNS_NO_MORE_ENTRIES                (-182)  /* No more entries be found.                            */
                                                      
#define SL_NET_APP_DNS_MAX_SERVICES_ERROR             (-200)  /* Maximum advertise services are already configured    */
#define SL_NET_APP_DNS_IDENTICAL_SERVICES_ERROR       (-201)  /* Trying to register a service that is already exists  */
#define SL_NET_APP_DNS_NOT_EXISTED_SERVICE_ERROR      (-203)  /* Trying to delete service that does not existed       */
#define SL_NET_APP_DNS_ERROR_SERVICE_NAME_ERROR       (-204)  /* Illegal service name according to the RFC            */
#define SL_NET_APP_DNS_RX_PACKET_ALLOCATION_ERROR     (-205)  /* Retry request                                        */
#define SL_NET_APP_DNS_BUFFER_SIZE_ERROR              (-206)  /* List size buffer is bigger than internally allowed in the NWP */
#define SL_NET_APP_DNS_NET_APP_SET_ERROR              (-207)  /* Illegal length of one of the mDNS Set functions      */
#define SL_NET_APP_DNS_GET_SERVICE_LIST_FLAG_ERROR    (-208)
#define SL_NET_APP_DNS_NO_CONFIGURATION_ERROR         (-209)

/* Set Dev name error codes  (NETAPP_SET_GET_DEV_CONF_OPT_DEVICE_URN) */
#define SL_ERROR_DEVICE_NAME_LEN_ERR                   (-117) 
#define SL_ERROR_DEVICE_NAME_INVALID                   (-118)
/* Set domain name error codes (NETAPP_SET_GET_DEV_CONF_OPT_DOMAIN_NAME) */
#define SL_ERROR_DOMAIN_NAME_LEN_ERR                   (-119)
#define SL_ERROR_DOMAIN_NAME_INVALID                   (-120)

/******************************************************************************/

/* NetApp application IDs */
#define SL_NET_APP_HTTP_SERVER_ID                     (1)
#define SL_NET_APP_DHCP_SERVER_ID                     (2)
#define SL_NET_APP_MDNS_ID                            (4)
#define SL_NET_APP_DNS_SERVER_ID                      (8)
#define SL_NET_APP_DEVICE_CONFIG_ID                   (16)
/* NetApp application set/get options */             
#define NETAPP_SET_DHCP_SRV_BASIC_OPT                 (0)             
/* HTTP server set/get options */                    
#define NETAPP_SET_GET_HTTP_OPT_PORT_NUMBER           (0)
#define NETAPP_SET_GET_HTTP_OPT_AUTH_CHECK            (1)
#define NETAPP_SET_GET_HTTP_OPT_AUTH_NAME             (2)
#define NETAPP_SET_GET_HTTP_OPT_AUTH_PASSWORD         (3)
#define NETAPP_SET_GET_HTTP_OPT_AUTH_REALM            (4)
#define NETAPP_SET_GET_HTTP_OPT_ROM_PAGES_ACCESS      (5)
                                                     
#define NETAPP_SET_GET_MDNS_CONT_QUERY_OPT            (1)
#define NETAPP_SET_GET_MDNS_QEVETN_MASK_OPT           (2)
#define NETAPP_SET_GET_MDNS_TIMING_PARAMS_OPT         (3)

/* DNS server set/get options */
#define NETAPP_SET_GET_DNS_OPT_DOMAIN_NAME            (0)

/* Device Config set/get options */
#define NETAPP_SET_GET_DEV_CONF_OPT_DEVICE_URN        (0)
#define NETAPP_SET_GET_DEV_CONF_OPT_DOMAIN_NAME       (1)


/*****************************************************************************/
/* Structure/Enum declarations                                               */
/*****************************************************************************/

typedef struct
{
    UINT32    PacketsSent;
    UINT32    PacketsReceived;
    UINT16    MinRoundTime;
    UINT16    MaxRoundTime;
    UINT16    AvgRoundTime;
    UINT32    TestTime;
}SlPingReport_t;

typedef struct
{
    UINT32    PingIntervalTime;       /* delay between pings, in milliseconds */
    UINT16    PingSize;               /* ping packet size in bytes           */
    UINT16    PingRequestTimeout;     /* timeout time for every ping in milliseconds  */
    UINT32    TotalNumberOfAttempts;  /* max number of ping requests. 0 - forever    */
    UINT32    Flags;                  /* flag - 0 report only when finished, 1 - return response for every ping, 2 - stop after 1 successful ping.  */
    UINT32    Ip;                     /* IPv4 address or IPv6 first 4 bytes  */
    UINT32    Ip1OrPaadding;
    UINT32    Ip2OrPaadding;
    UINT32    Ip3OrPaadding;
}SlPingStartCommand_t;

typedef struct _slHttpServerString_t
{
    UINT8     len;
    UINT8     *data;
} slHttpServerString_t;

typedef struct _slHttpServerData_t
{
    UINT8     value_len;
    UINT8     name_len;
    UINT8     *token_value;
    UINT8     *token_name;
} slHttpServerData_t;

typedef struct _slHttpServerPostData_t
{
    slHttpServerString_t action;
    slHttpServerString_t token_name;
    slHttpServerString_t token_value;
}slHttpServerPostData_t;

typedef union
{
  slHttpServerString_t   httpTokenName; //SL_NETAPP_HTTPGETTOKENVALUE
  slHttpServerPostData_t httpPostData;  //SL_NETAPP_HTTPPOSTTOKENVALUE
} SlHttpServerEventData_u;

typedef union
{
  slHttpServerString_t token_value;
} SlHttpServerResponsedata_u;

typedef struct
{
   UINT32  Event;
   SlHttpServerEventData_u EventData;
}SlHttpServerEvent_t;

typedef struct
{
   UINT32  Response;
   SlHttpServerResponsedata_u ResponseData;
}SlHttpServerResponse_t;

typedef struct
{
    UINT32   lease_time;
    UINT32   ipv4_addr_start;
    UINT32   ipv4_addr_last;
}SlNetAppDhcpServerBasicOpt_t; 

/*mDNS parameters*/
typedef enum
{
    SL_NET_APP_FULL_SERVICE_WITH_TEXT_IPV4_TYPE = 1,
    SL_NET_APP_FULL_SERVICE_IPV4_TYPE,
    SL_NET_APP_SHORT_SERVICE_IPV4_TYPE
 
} SlNetAppGetServiceListType_e;

typedef struct
{
    UINT32   service_ipv4;
    UINT16   service_port;
    UINT16   Reserved;
}SlNetAppGetShortServiceIpv4List_t;

typedef struct
{
    UINT32   service_ipv4;
    UINT16   service_port;
    UINT16   Reserved;
    UINT8    service_name[NETAPP_MAX_SERVICE_NAME_SIZE];
    UINT8    service_host[NETAPP_MAX_SERVICE_HOST_NAME_SIZE];
}SlNetAppGetFullServiceIpv4List_t;

typedef struct
{
    UINT32    service_ipv4;
    UINT16    service_port;
    UINT16    Reserved;
    UINT8     service_name[NETAPP_MAX_SERVICE_NAME_SIZE];
    UINT8     service_host[NETAPP_MAX_SERVICE_HOST_NAME_SIZE];
    UINT8     service_text[NETAPP_MAX_SERVICE_TEXT_SIZE];
}SlNetAppGetFullServiceWithTextIpv4List_t;

typedef struct
{
    /*The below parameters are used to configure the advertise times and interval
    For example:
        If:
        Period is set to T
        Repetitions are set to P
        Telescopic factor is K=2
        The transmission shall be:
        advertise P times
        wait T
        advertise P times
        wait 4 * T
        advertise P time
        wait 16 * T  ... (till max time reached / configuration changed / query issued)
    */
    UINT32    t;              /* Number of ticks for the initial period. Default is 100 ticks for 1 second. */
    UINT32    p;              /* Number of repetitions. Default value is 1                                  */
    UINT32    k;              /* Telescopic factor. Default value is 2.                                     */
    UINT32    RetransInterval;/* Announcing retransmission interval                                         */
    UINT32    Maxinterval;     /* Announcing max period interval                                            */
    UINT32    max_time;       /* Announcing max time                                                        */
}SlNetAppServiceAdvertiseTimingParameters_t;

/*****************************************************************************/
/* Types declarations                                               */
/*****************************************************************************/
typedef void (*P_SL_DEV_PING_CALLBACK)(SlPingReport_t*);

/*****************************************************************************/
/* Function prototypes                                                       */
/*****************************************************************************/
#if _SL_INCLUDE_FUNC(sl_NetAppStart)
INT16 sl_NetAppStart(const UINT32 AppBitMap);
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppStop)
INT16 sl_NetAppStop(const UINT32 AppBitMap);
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppGetServiceList)
INT16 sl_NetAppGetServiceList
        (const UINT8 IndexOffest, const UINT8 MaxServiceCount,
         const UINT8 Flags, INT8 *pBuffer, const UINT32 RxBufferLength);
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppMDNSRegisterService)
INT16 sl_NetAppMDNSRegisterService
        ( const INT8* pServiceName, const UINT8 ServiceNameLen, const INT8* pText,
          const UINT8 TextLen, const UINT16 Port, const UINT32 TTL, UINT32 Options);
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppMDNSUnRegisterService)
INT16 sl_NetAppMDNSUnRegisterService(const INT8 *pServiceName,const UINT8 ServiceNameLen);
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppDnsGetHostByService)
INT32 sl_NetAppDnsGetHostByService
        (INT8 *pServiceName, const UINT8 ServiceLen, const UINT8 Family,
         UINT32 pAddr[], UINT32 *pPort, UINT16 *pTextLen, INT8 *pText);
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppDnsGetHostByName)
INT16 sl_NetAppDnsGetHostByName
        (INT8 * hostname,const  UINT16 usNameLen,
         UINT32* out_ip_addr, const UINT8 family );
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppPingStart)
INT16 sl_NetAppPingStart
            (const SlPingStartCommand_t* pPingParams,const UINT8 family,
             SlPingReport_t *pReport, const P_SL_DEV_PING_CALLBACK pPingCallback);
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppSet)
INT32 sl_NetAppSet(const UINT8 AppId ,const UINT8 Option,const
                         UINT8 OptionLen,const UINT8 *pOptionValue);
#endif

#if _SL_INCLUDE_FUNC(sl_NetAppGet)
INT32 sl_NetAppGet(const UINT8 AppId,const  UINT8 Option,
                         UINT8 *pOptionLen, UINT8 *pOptionValue);
#endif


#ifdef  __cplusplus
}
#endif /*  __cplusplus */

#endif    /*  __NETAPP_H__ */

