/*
 * netcfg.h - CC31xx/CC32xx Host Driver Implementation
 *
 * Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/*****************************************************************************/
/* Include files                                                             */
/*****************************************************************************/
#include "CC3100/include/simplelink.h"

    
#ifndef __NETCFG_H__
#define __NETCFG_H__


#ifdef    __cplusplus
extern "C" {
#endif

/*****************************************************************************/
/* Macro declarations                                                        */
/*****************************************************************************/

#define SL_MAC_ADDR_LEN                          (6)
#define SL_IPV4_VAL(add_3,add_2,add_1,add_0)     ((((UINT32)add_3 << 24) & 0xFF000000) | (((UINT32)add_2 << 16) & 0xFF0000) | (((UINT32)add_1 << 8) & 0xFF00) | ((UINT32)add_0 & 0xFF) )
#define SL_IPV4_BYTE(val,index)                  ( (val >> (index*8)) & 0xFF )

#define IPCONFIG_MODE_DISABLE_IPV4               (0)
#define IPCONFIG_MODE_ENABLE_IPV4                (1)

/*****************************************************************************/
/* Structure/Enum declarations                                               */
/*****************************************************************************/
typedef enum
{
    SL_MAC_ADDRESS_SET          		= 1,
    SL_MAC_ADDRESS_GET          		= 2,
    SL_IPV4_STA_P2P_CL_GET_INFO         = 3,
    SL_IPV4_STA_P2P_CL_DHCP_ENABLE      = 4,
    SL_IPV4_STA_P2P_CL_STATIC_ENABLE    = 5,
    SL_IPV4_AP_P2P_GO_GET_INFO          = 6,
    SL_IPV4_AP_P2P_GO_STATIC_ENABLE     = 7,
    SL_SET_HOST_RX_AGGR                 = 8,
    MAX_SETTINGS = 0xFF
}Sl_NetCfg_e;

typedef struct
{
    UINT32  ipV4;
    UINT32  ipV4Mask;
    UINT32  ipV4Gateway;
    UINT32  ipV4DnsServer;
}SlNetCfgIpV4Args_t;

/*******************************************************************************
 * Function prototypes
 ******************************************************************************/
#if _SL_INCLUDE_FUNC(sl_NetCfgSet)
INT32 sl_NetCfgSet(const UINT8 ConfigId,const UINT8 ConfigOpt,const UINT8 ConfigLen,const UINT8 *pValues);
#endif

#if _SL_INCLUDE_FUNC(sl_NetCfgGet)
INT32 sl_NetCfgGet(const UINT8 ConfigId ,UINT8 *pConfigOpt, UINT8 *pConfigLen, UINT8 *pValues);
#endif


#ifdef  __cplusplus
}
#endif /*  __cplusplus */

#endif    /*  __NETCFG_H__ */

