/*
    FreeRTOS V8.2.2 - Copyright (C) 2015 Real Time Engineers Ltd.
    All rights reserved

    VISIT http://www.FreeRTOS.org TO ENSURE YOU ARE USING THE LATEST VERSION.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation >>!AND MODIFIED BY!<< the FreeRTOS exception.

    ***************************************************************************
    >>!   NOTE: The modification to the GPL is included to allow you to     !<<
    >>!   distribute a combined work that includes FreeRTOS without being   !<<
    >>!   obliged to provide the source code for proprietary components     !<<
    >>!   outside of the FreeRTOS kernel.                                   !<<
    ***************************************************************************

    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT ANY
    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
    FOR A PARTICULAR PURPOSE.  Full license text is available on the following
    link: http://www.freertos.org/a00114.html

    ***************************************************************************
     *                                                                       *
     *    FreeRTOS provides completely free yet professionally developed,    *
     *    robust, strictly quality controlled, supported, and cross          *
     *    platform software that is more than just the market leader, it     *
     *    is the industry's de facto standard.                               *
     *                                                                       *
     *    Help yourself get started quickly while simultaneously helping     *
     *    to support the FreeRTOS project by purchasing a FreeRTOS           *
     *    tutorial book, reference manual, or both:                          *
     *    http://www.FreeRTOS.org/Documentation                              *
     *                                                                       *
    ***************************************************************************

    http://www.FreeRTOS.org/FAQHelp.html - Having a problem?  Start by reading
    the FAQ page "My application does not run, what could be wrong?".  Have you
    defined configASSERT()?

    http://www.FreeRTOS.org/support - In return for receiving this top quality
    embedded software for free we request you assist our global community by
    participating in the support forum.

    http://www.FreeRTOS.org/training - Investing in training allows your team to
    be as productive as possible as early as possible.  Now you can receive
    FreeRTOS training directly from Richard Barry, CEO of Real Time Engineers
    Ltd, and the world's leading authority on the world's leading RTOS.

    http://www.FreeRTOS.org/plus - A selection of FreeRTOS ecosystem products,
    including FreeRTOS+Trace - an indispensable productivity tool, a DOS
    compatible FAT file system, and our tiny thread aware UDP/IP stack.

    http://www.FreeRTOS.org/labs - Where new FreeRTOS products go to incubate.
    Come and try FreeRTOS+TCP, our new open source TCP/IP stack for FreeRTOS.

    http://www.OpenRTOS.com - Real Time Engineers ltd. license FreeRTOS to High
    Integrity Systems ltd. to sell under the OpenRTOS brand.  Low cost OpenRTOS
    licenses offer ticketed support, indemnification and commercial middleware.

    http://www.SafeRTOS.com - High Integrity Systems also provide a safety
    engineered and independently SIL3 certified version for use in safety and
    mission critical applications that require provable dependability.

    1 tab == 4 spaces!
*******************************************************************************/
//Uart1 app include files.
#include "FreeRTOS/Uart1/Uart1.h"

/*******************************************************************************
 * Desc Opens uart1 and enables the interrupt, and creates TX and RX queues.
 * Parm ulWantedBaud - Port baud rate. EX: 115200ul
 * Parm uxQueueLength - leangth of the TX and RX Queues
 * Return None
 ******************************************************************************/
void xSerialPortInitMinimal( unsigned long ulWantedBaud, unsigned portBASE_TYPE uxQueueLength )
{
    unsigned short usBRG;

    //Create the queues used by the com test task./
    xRxedChars = xQueueCreate( uxQueueLength, ( unsigned portBASE_TYPE ) sizeof( signed char ) );
    xCharsForTx = xQueueCreate( uxQueueLength, ( unsigned portBASE_TYPE ) sizeof( signed char ) );

    //create Rx '\n' recieved semaphore
    xRXreturnSemaphore = xSemaphoreCreateBinary();

    //Configure the UART and interrupts.
    usBRG = (unsigned short)(( (float)configPERIPHERAL_CLOCK_HZ / ( (float)16 * (float)ulWantedBaud ) ) - (float)0.5);
    OpenUART1( UART_EN, UART_RX_ENABLE | UART_TX_ENABLE | UART_INT_TX | UART_INT_RX_CHAR, usBRG );
    ConfigIntUART1( ( configKERNEL_INTERRUPT_PRIORITY + 1 ) | UART_INT_SUB_PR0 | UART_RX_INT_EN );

    xTxHasEnded = pdTRUE;

    return;
}

/*******************************************************************************
 * Desc Function to close uart1
 * Parm None
 * Return None
 ******************************************************************************/
void vSerialClose( void )
{
    U1MODEbits.ON = 0;
}

/*******************************************************************************
 * The UART interrupt handler.  As this uses the FreeRTOS assembly interrupt
 * entry point the IPL setting in the following prototype has no effect.  The
 * interrupt priority is set by the call to  ConfigIntUART2() in
 * xSerialPortInitMinimal().
 ******************************************************************************/
void __attribute__( (interrupt(IPL0AUTO), vector(_UART1_VECTOR))) vU1InterruptWrapper( void );

/*******************************************************************************
 * Desc Interrupt handler for both Tx and RX interrups on uart1.
 * Parm None
 * Return None
 ******************************************************************************/
void vU1InterruptHandler( void )
{
    //Declared static to minimise stack use.
    static char cChar;
    static portBASE_TYPE xHigherPriorityTaskWoken;

    xHigherPriorityTaskWoken = pdFALSE;

    //Are any Rx interrupts pending?
    if( IFS0bits.U1RXIF == 1)
    {
	while( U1STAbits.URXDA )
	{
            //Retrieve the received character and place it in the queue of
            //received characters.
            cChar = U1RXREG;
            xQueueSendFromISR( xRxedChars, &cChar, &xHigherPriorityTaskWoken );
             if(cChar == '\n')
             {
                //Unblock the task by giving the semaphore.
                xSemaphoreGiveFromISR(xRXreturnSemaphore, &xHigherPriorityTaskWoken );
             }
	}
	IFS0CLR = _IFS0_U1RXIF_MASK;
    }

    //Are any Tx interrupts pending?
    if( IFS0bits.U1TXIF == 1 )
    {
	while( ( U1STAbits.UTXBF ) == 0 )
	{
            if( xQueueReceiveFromISR( xCharsForTx, &cChar, &xHigherPriorityTaskWoken ) == pdTRUE )
            {
		//Send the next character queued for Tx.
		U1TXREG = cChar;
            }
            else
            {
		//Queue empty, nothing to send.
		xTxHasEnded = pdTRUE;
                IEC0CLR = _IEC0_U1TXIE_MASK;
		break;
            }
	}

	IFS0CLR = _IFS0_U1TXIF_MASK;
    }

    //If sending or receiving necessitates a context switch, then switch now.
    portEND_SWITCHING_ISR( xHigherPriorityTaskWoken );
}

/*******************************************************************************
 * Desc Places a byte in the transmit buffer and enables the TX interrupt for uart1
 * Parm cOutChar - Byte to put on buffer
 * Parm xBlockTime - Time to wait for buffer to empty
 * Return None
 ******************************************************************************/
signed portBASE_TYPE xSerialPutChar(signed char cOutChar, TickType_t xBlockTime )
{
    signed portBASE_TYPE xReturn;

    //Return false if after the block time there is no room on the Tx queue.
    if( xQueueSend( xCharsForTx, &cOutChar, xBlockTime ) != pdPASS )
    {
	xReturn = pdFAIL;
    }
    else
    {
	xReturn = pdPASS;
    }

    if( xReturn != pdFAIL )
    {
	//A critical section should not be required as xTxHasEnded will not be
	//written to by the ISR if it is already 0.
	if(  xTxHasEnded == pdTRUE )
	{
            xTxHasEnded = pdFALSE;
            IEC0SET = _IEC0_U1TXIE_MASK;
	}
    }

    return pdPASS;
}

/*******************************************************************************
 * Desc Copys cpOutString to the transmit buffer and enables the TX interrupt for uart1
 * Parm cpOutString - Pointer to string to be copyed. String must have null terminator.
 * Parm xBlockTime - Time to wait for buffer to empty
 * Return None
 ******************************************************************************/
signed portBASE_TYPE xSerialPutString(signed char *cpOutString, TickType_t xBlockTime )
{
    signed portBASE_TYPE xReturn;
    unsigned char usLength = strlen((const char *)cpOutString);

    while (usLength)
    {
        //Return false if after the block time there is no room on the Tx queue.
        if( xQueueSend( xCharsForTx, cpOutString, xBlockTime ) != pdPASS )
        {
            xReturn = pdFAIL;
        }
        else
        {
            xReturn = pdPASS;
        }

        usLength--;
        cpOutString++;
    }

    if( xReturn != pdFAIL )
    {
	//A critical section should not be required as xTxHasEnded will not be
	//written to by the ISR if it is already 0.
	if(  xTxHasEnded == pdTRUE )
	{
            xTxHasEnded = pdFALSE;
            IEC0SET = _IEC0_U1TXIE_MASK;
	}
    }

    return pdPASS;
}

/*******************************************************************************
 * Desc Gets a byte from RX queue and places it at the pcRXedchar pointer
 * Parm pcRxedChar - pointer to location to store byte
 * Parm xBlockTime - Time to wait for buffer to empty
 * Return None
 ******************************************************************************/
signed portBASE_TYPE xSerialGetChar( signed char *pcRxedChar, TickType_t xBlockTime )
{
	/* Get the next character from the buffer.  Return false if no characters
	are available or arrive before xBlockTime expires. */
	if( xQueueReceive( xRxedChars, pcRxedChar, xBlockTime ) )
	{
		return pdTRUE;
	}
	else
	{
		return pdFALSE;
	}
}




