/*******************************************************************************
 * FileName:        Max32USB.h
 * Dependencies:    plib.h
 * Processor:       PIC32
 * Compiler:        CX32
 * Company:         Pacific Coast Motion
 * Author:          Thomas McGee
 * Created:         June 23, 2012, 1:30 PM
*******************************************************************************/
#include "Max32USB/Max32USB.h"

/*******************************************************************************
*  Function Name:  OpenMaxUSB(Bus Clock Speed, dataRate)                     
*  Return Value:   void                                                      
*  Parameters:     sourceClock = GetPeripheralClock()                        
*                  dataRate = Baud rate                                      
*  Description:    Opens and configures micro USB port of Max32.             
*******************************************************************************/
void OpenMaxUSB(UINT32 sourceClock, UINT32 dataRate )
{
    UART_CONFIGURATION flags;

    if(dataRate >= 57600L)
        {
        flags = UART_ENABLE_PINS_TX_RX_ONLY|UART_ENABLE_HIGH_SPEED;
        }
    else
        {
        flags = UART_ENABLE_PINS_TX_RX_ONLY;
        }
    
    UARTConfigure(UART1, flags);
    UARTSetFifoMode(UART1, UART_INTERRUPT_ON_TX_NOT_FULL | UART_INTERRUPT_ON_RX_NOT_EMPTY);
    UARTSetLineControl(UART1, UART_DATA_SIZE_8_BITS | UART_PARITY_NONE | UART_STOP_BITS_1);
    UARTSetDataRate(UART1, sourceClock, dataRate);
    UARTEnable(UART1, UART_ENABLE_FLAGS(UART_PERIPHERAL | UART_RX | UART_TX));  
}

/*******************************************************************************
*  Function Name:  void PutUSBbyte(void)
*  Return Value:   void
*  Parameters:     byte to transmit
*  Description:    transmits a byte out the micro USB port.
*******************************************************************************/
void PutUSBbyte(UINT8 Data)
{
    while(!UARTTransmitterIsReady(UART1));

    UARTSendDataByte(UART1, Data);

}

/*******************************************************************************
*  Function Name:  void PutUSBarray(const char *buffer, UINT32 size)        
*  Return Value:   void                                                      
*  Parameters:     pointer to array, size of array.
*  Description:    transmits an array of "size" out the USB port.        
*******************************************************************************/
void PutUSBarray(const char *buffer, int size)
{
    while(size)
    {
        while(!UARTTransmitterIsReady(UART1));

        UARTSendDataByte(UART1, *buffer);

        buffer++;
        size--;
    }

    while(!UARTTransmissionHasCompleted(UART1));
}
/*******************************************************************************
*  Function Name:  void PutUSBString(const char *buffer)                   
*  Return Value:   void                                                     
*  Parameters:     pointer to array of ASCII, must be trminated with '/0'(0x00).
*  Description:    Transmits a string out the USB port, end at '/0'        
*******************************************************************************/
void PutUSBstring(const char *buffer)
{
    while(*buffer != 0x00)
    {
        while(!UARTTransmitterIsReady(UART1));
        UARTSendDataByte(UART1, *buffer);
        buffer++;
        
    }

    while(!UARTTransmissionHasCompleted(UART1));
}

// *****************************************************************************
// UINT32 GetUSBbyte(void)
// *****************************************************************************
UINT8 GetUSBbyte(void)
{
    UINT8  Data;

    while(!UARTReceivedDataIsAvailable(UART1));

    Data = UARTGetDataByte(UART1);

    return Data;
}

// *****************************************************************************
// UINT32 GetUSBarray(char *buffer, UINT32 max_size)
// *****************************************************************************
UINT32 GetUSBarray(char *buffer, int max_size)
{
    UINT32 num_char;
    UINT8 character;

    num_char = 0;

    while(num_char < max_size )
    {

        while(!UARTReceivedDataIsAvailable(UART1));

        character = UARTGetDataByte(UART1);

        if(character == '\r')break;

        *buffer = character;

        buffer++;

        num_char++;

    }

    return num_char;
}

void menuUSBchoice(void)
{
    UINT8 menu_choice;

    menu_choice = GetUSBbyte();

    switch(menu_choice)
        {
        case 1:
             // Do something
            
            break;

        case 2:
            // Do something
            
            break;

        case 3:
            // Do something
            
            break;

        case 4:
             // Do something
            
            break;

        case 5:
            // Do something
            
            break;

        case 6:
            // Do something
           
            break;

        case 7:
            // Do something
            
            break;

        case 8:
            // Do something
            
            break;

        case 9:
            // Do something
            
            break;

        case 10:
            // Do something
           
            break;

        case 11:
            // Do something
           
            break;

        case 12:
            // Do something
           
            break;

        case 13:
            // Do something
          
            break;

        case 14:
            // Do something
          
            break;

        }

}

