/* 
 * File:   SDcard_fat_sl.h
 * Author: Tower
 *
 * Created on October 2, 2015, 12:16 PM
 */
#ifndef SDCARD_FAT_SL_H
#define	SDCARD_FAT_SL_H

/**** Labels SDcard_drv_f *****************************************************/
#define F_SECTOR_SIZE           512u  /* Disk sector size. */
#define F_FS_THREAD_AWARE       0     /* Set to one if the file system will be access from more than one task. */
#define F_MAXPATH               64    /* Maximum length a file name (including its full path) can be. */
#define F_MAX_LOCK_WAIT_TICKS   20    /* The maximum number of RTOS ticks to wait when attempting to obtain a lock on the file system when F_FS_THREAD_AWARE is set to 1. */

#define F_RAM_DRIVE0            0
#define F_MEDIADESC_REMOVABLE   0xf0
#define F_MEDIADESC_FIX         0xf8

#define F_ST_MOUNTED            0x0000
#define F_ST_MISSING            0x0001
#define F_ST_CHANGED            0x0002
#define F_ST_WRPROTECT          0x0004

/**** Labels dir **************************************************************/
/* definitions for ctime */
#define F_CTIME_SEC_SHIFT       0
#define F_CTIME_SEC_MASK        0x001f  /*0-30 in 2seconds*/
#define F_CTIME_MIN_SHIFT       5
#define F_CTIME_MIN_MASK        0x07e0  /*0-59 */
#define F_CTIME_HOUR_SHIFT      11
#define F_CTIME_HOUR_MASK       0xf800  /*0-23*/

/* definitions for cdate */
#define F_CDATE_DAY_SHIFT       0
#define F_CDATE_DAY_MASK        0x001f  /*0-31*/
#define F_CDATE_MONTH_SHIFT     5
#define F_CDATE_MONTH_MASK      0x01e0  /*1-12*/
#define F_CDATE_YEAR_SHIFT      9
#define F_CDATE_YEAR_MASK       0xfe00  /*0-119 (1980+value)*/

#define NTRES_LOW               0x08 /*lower case name*/

/**** Labels fat_sl ***********************************************************/
#define F_MAXNAME               8    /* 8 byte name */
#define F_MAXEXT                3    /* 3 byte extension */

#define F_ATTR_ARC              0x20
#define F_ATTR_DIR              0x10
#define F_ATTR_VOLUME           0x08
#define F_ATTR_SYSTEM           0x04
#define F_ATTR_HIDDEN           0x02
#define F_ATTR_READONLY         0x01

#define F_CLUSTER_FREE          ((unsigned long)0x00000000 )
#define F_CLUSTER_RESERVED      ((unsigned long)0x0ffffff0 )
#define F_CLUSTER_BAD           ((unsigned long)0x0ffffff7 )
#define F_CLUSTER_LAST          ((unsigned long)0x0ffffff8 )
#define F_CLUSTER_LASTF32R      ((unsigned long)0x0fffffff )

/**** Labels file** ***********************************************************/
#define F_FILE_CLOSE            0x00
#define F_FILE_RD               0x01
#define F_FILE_WR               0x02
#define F_FILE_A                0x04
#define F_FILE_RDP              0x08
#define F_FILE_WRP              0x10
#define F_FILE_AP               0x20

/**** Labels psp_string.h *****************************************************/
#define psp_memcpy( d, s, l )    memcpy( ( d ), ( s ), (size_t)( l ) )
#define psp_memmove( d, s, l )   memmove( ( d ), ( s ), (size_t)( l ) )
#define psp_memset( d, c, l )    memset( ( d ), ( c ), (size_t)( l ) )
#define psp_memcmp( s1, s2, l )  memcmp( ( s1 ), ( s2 ), (size_t)( l ) )
#define psp_strnlen( s, l )      strnlen( ( s ), ( size_t )( l ) )
#define psp_strncat( d, s, l )   strncat( ( d ), ( s ), (size_t)( l ) )
#define psp_strncpy( d, s, l )   strncpy( ( d ), ( s ), (size_t)( l ) )
#define psp_strncmp( s1, s2, l ) strncmp( ( s1 ), ( s2 ), (size_t)( l ) )

/**** Typedefs api_mdriver_flash **********************************************/
typedef struct
{
  unsigned short  number_of_cylinders;
  unsigned short  sector_per_track;
  unsigned short  number_of_heads;
  unsigned long   number_of_sectors;
  unsigned char   media_descriptor;
  unsigned short  bytes_per_sector;
} F_PHY;

typedef struct F_DRIVER  F_DRIVER;

typedef int  ( *F_WRITESECTOR )( F_DRIVER * driver, char* data, unsigned long sector );
typedef int  ( *F_READSECTOR )( F_DRIVER * driver, char* data, unsigned long sector );
typedef int  ( *F_GETPHY )( F_DRIVER * driver, F_PHY * phy );
typedef int  ( *F_GETSTATUS )( F_DRIVER * driver );
typedef void ( *F_RELEASE )( F_DRIVER * driver );

typedef struct F_DRIVER
{
    unsigned long  user_data;     /* user defined data */
    void*          user_ptr;      /* user define pointer */

    /* driver functions */
    F_WRITESECTOR writesector;
    F_READSECTOR  readsector;
    F_GETPHY      getphy;
    F_GETSTATUS   getstatus;
    F_RELEASE     release;
} _F_DRIVER;

typedef INT16 ( *F_DRIVERINIT )( unsigned long driver_param );
typedef INT16 ( *response_type)( UINT8 RespTok );

enum
{
    SD_CARD_NO_ERROR,
    SD_CARD_ERR_SECTOR = 101,
    SD_CARD_ERR_NOTAVAILABLE
};

/**** Typedefs dir ************************************************************/
typedef struct
{
  unsigned char  name[F_MAXNAME];   /* 8+3 */
  unsigned char  ext[F_MAXEXT];
  unsigned char  attr;              /* 00ADVSHR */

  unsigned char  ntres;             /* FAT32 only  */
  unsigned char  crttimetenth;      /* FAT32 only  */
  unsigned char  crttime[2];        /* FAT32 only  */
  unsigned char  crtdate[2];        /* FAT32 only */
  unsigned char  lastaccessdate[2]; /* FAT32 only */

  unsigned char  clusterhi[2];      /* FAT32 only */
  unsigned char  ctime[2];
  unsigned char  cdate[2];
  unsigned char  clusterlo[2];      /* fat12,fat16,fat32 */
  unsigned char  filesize[4];
} F_DIRENTRY;

/**** Typedefs fat_sl *********************************************************/
typedef struct
{
  char  path[F_MAXPATH];        /*  /directory1/dir2/  */
  char  filename[F_MAXNAME];    /* filename */
  char  fileext[F_MAXEXT];      /* extension */
} F_NAME;

typedef struct
{
  unsigned long  cluster;
  unsigned long  sector;
  unsigned long  sectorend;
  unsigned long  pos;
} F_POS;

typedef struct
{
  char            filename[F_MAXPATH]; /*file name+ext*/
  char            name[F_MAXNAME];     /*file name*/
  char            ext[F_MAXEXT];       /*file extension*/
  unsigned char   attr;                /*attribute of the file*/

  unsigned short  ctime;        /*creation time*/
  unsigned short  cdate;        /*creation date*/
  unsigned long   cluster;

  long            filesize;         /*length of file*/

  F_NAME          findfsname;   /*find properties*/
  F_POS           pos;
} F_FIND;

typedef struct
{
  unsigned long  abspos;
  unsigned long  filesize;
  unsigned long  startcluster;
  unsigned long  relpos;
  unsigned char  modified;
  unsigned char  mode;
  unsigned char  _tdata[F_SECTOR_SIZE];
  F_POS          pos;
  F_POS          dirpos;
#if F_FILE_CHANGED_EVENT
  char           filename[F_MAXPATH];   /* filename with full path */
#endif
} F_FILE;

enum
{
  F_UNKNOWN_MEDIA
  , F_FAT12_MEDIA
  , F_FAT16_MEDIA
  , F_FAT32_MEDIA
};

enum
{
    F_NO_ERROR,             //0
    F_ERR_RESERVED_1,       //1
    F_ERR_NOTFORMATTED,     //2
    F_ERR_INVALIDDIR,       //3
    F_ERR_INVALIDNAME,      //4
    F_ERR_NOTFOUND,         //5
    F_ERR_DUPLICATED,       //6
    F_ERR_NOMOREENTRY,      //7
    F_ERR_NOTOPEN,          //8
    F_ERR_EOF,              //9
    F_ERR_RESERVED_2,       //10
    F_ERR_NOTUSEABLE,       //11
    F_ERR_LOCKED,           //12
    F_ERR_ACCESSDENIED,     //13
    F_ERR_NOTEMPTY,         //14
    F_ERR_INITFUNC,         //15
    F_ERR_CARDREMOVED,      //16
    F_ERR_ONDRIVE,          //17
    F_ERR_INVALIDSECTOR,    //18
    F_ERR_READ,             //19
    F_ERR_WRITE,            //20
    F_ERR_INVALIDMEDIA,     //21
    F_ERR_BUSY,             //22
    F_ERR_WRITEPROTECT,     //23
    F_ERR_INVFATTYPE,       //24
    F_ERR_MEDIATOOSMALL,    //25
    F_ERR_MEDIATOOLARGE,    //26
    F_ERR_NOTSUPPSECTORSIZE,//27
    F_ERR_ALLOCATION        //28
#if F_FS_THREAD_AWARE == 1
    , F_ERR_OS = 29         //29
#endif /* F_FS_THREAD_AWARE */
};

typedef struct
{
    unsigned long  total;
    unsigned long  free;
    unsigned long  used;
    unsigned long  bad;

    unsigned long  total_high;
    unsigned long  free_high;
    unsigned long  used_high;
    unsigned long  bad_high;
} F_SPACE;

enum
{
    F_SEEK_SET,     //Beginning of file
    F_SEEK_CUR,     //Current position of file pointer
    F_SEEK_END      //End of file
};

/**** Typedefs psp_rtc.h ******************************************************/
typedef struct
{
  unsigned char  sec;
  unsigned char  min;
  unsigned char  hour;
  unsigned char  day;
  unsigned char  month;
  unsigned short year;
} t_psp_timedate;

/**** Typedefs volume.h *******************************************************/
typedef struct
{
  unsigned char  sector_per_cluster;
  unsigned char  number_of_FATs;
  unsigned char  media_descriptor;
  unsigned long  rootcluster;
  unsigned long  sector_per_FAT;
  unsigned long  sector_per_FAT32;
  unsigned long  serial_number;
} F_BOOTRECORD;


typedef struct
{
  unsigned long  sector; /*start sector*/
  unsigned long  num;    /*number of sectors*/
} F_SECTOR;


typedef struct
{
  unsigned char  state;
  F_BOOTRECORD   bootrecord;
  F_SECTOR       firstfat;
  F_SECTOR       root;
  F_SECTOR       _tdata;

  unsigned long  actsector;
  unsigned long  fatsector;

  unsigned long  lastalloccluster;
  unsigned char  modified;
  char           cwd[F_MAXPATH]; /*current working folder in this volume*/
  unsigned char  mediatype;
  unsigned long  maxcluster;
} F_VOLUME;

enum
{
    F_STATE_NONE,       //0
    F_STATE_NEEDMOUNT,  //1
    F_STATE_WORKING     //2
};

/**** Globals drv *************************************************************/
F_DRIVER mdrv_structure;
F_DRIVER * mdrv;  //Pointer to driver structure
F_PHY phy;
char* pDataRead;
char* pDataWrite;
unsigned char sdhc;
unsigned long maxsector;


/**** Prototype api_mdriver_flash *********************************************/
INT16 SDCD_initfunc( unsigned long driver_param );

/**** Prototype dir ***********************************************************/
unsigned char _f_getdirsector ( unsigned long );
unsigned char _f_findfilewc ( char *, char *, F_POS *, F_DIRENTRY * *, unsigned char );
unsigned char _f_findpath ( F_NAME *, F_POS * );
unsigned long _f_getdecluster ( F_DIRENTRY * );

unsigned char _f_writedirsector ( void );
void _f_setdecluster ( F_DIRENTRY *, unsigned long );
unsigned char _f_addentry ( F_NAME *, F_POS *, F_DIRENTRY * * );

/**** Prototype drv ***********************************************************/
unsigned char _f_checkstatus ( void );
unsigned char _f_readglsector ( unsigned long );
unsigned char _f_writeglsector ( unsigned long );

/**** Prototype f_lock ********************************************************/
unsigned char fsr_init ( void );
unsigned char fsr_delete ( void );

/**** Prototype fat ********************************************************/
unsigned char _f_getfatsector ( unsigned long );
unsigned char _f_getclustervalue ( unsigned long, unsigned long * );
void _f_clustertopos ( unsigned long, F_POS * );
unsigned char _f_getcurrsector ( void );

unsigned char _f_writefatsector ( void );
unsigned char _f_setclustervalue ( unsigned long, unsigned long );
unsigned char _f_alloccluster ( unsigned long * );
unsigned char _f_removechain ( unsigned long );

/**** Prototype fat_sl for file changed events ********************************/
#if F_FILE_CHANGED_EVENT
typedef struct
{
  unsigned char   action;
  unsigned char   flags;
  unsigned char   attr;
  unsigned short  ctime;
  unsigned short  cdate;
  unsigned long   filesize;
  char            filename[F_MAXPATH];
} ST_FILE_CHANGED;

typedef void ( *F_FILE_CHANGED_EVENTFUNC )( ST_FILE_CHANGED * fc );

extern F_FILE_CHANGED_EVENTFUNC  f_filechangedevent;

 #define f_setfilechangedevent( filechangeevent ) f_filechangedevent = filechangeevent

/* flags */

 #define FFLAGS_NONE              0x00000000

 #define FFLAGS_FILE_NAME         0x00000001
 #define FFLAGS_DIR_NAME          0x00000002
 #define FFLAGS_NAME              0x00000003
 #define FFLAGS_ATTRIBUTES        0x00000004
 #define FFLAGS_SIZE              0x00000008
 #define FFLAGS_LAST_WRITE        0x00000010

/* actions */

 #define FACTION_ADDED            0x00000001
 #define FACTION_REMOVED          0x00000002
 #define FACTION_MODIFIED         0x00000003
 #define FACTION_RENAMED_OLD_NAME 0x00000004
 #define FACTION_RENAMED_NEW_NAME 0x00000005

#endif /* if F_FILE_CHANGED_EVENT */

unsigned char fs_init ( void );
unsigned char fs_delete ( void );

#define f_initvolume fn_initvolume
#define f_delvolume  fn_delvolume

unsigned char fn_initvolume ( F_DRIVERINIT initfunc );
unsigned char fn_delvolume ( void );

unsigned char fn_getfreespace ( F_SPACE * pspace );

unsigned char fn_chdir ( const char * dirname );
unsigned char fn_mkdir ( const char * dirname );
unsigned char fn_rmdir ( const char * dirname );

unsigned char fn_findfirst ( const char * filename, F_FIND * find );
unsigned char fn_findnext ( F_FIND * find );

long fn_filelength ( const char * filename );

unsigned char fn_close ( F_FILE * filehandle );
F_FILE * fn_open ( const char * filename, const char * mode );

long fn_read ( void * buf, long size, long _size_t, F_FILE * filehandle );

long fn_write ( const void * buf, long size, long _size_t, F_FILE * filehandle );

unsigned char fn_seek ( F_FILE * filehandle, long offset, unsigned char whence );

long fn_tell ( F_FILE * filehandle );
int fn_getc ( F_FILE * filehandle );
int fn_putc ( int ch, F_FILE * filehandle );
unsigned char fn_rewind ( F_FILE * filehandle );
unsigned char fn_eof ( F_FILE * filehandle );

unsigned char fn_delete ( const char * filename );

unsigned char fn_seteof ( F_FILE * );

F_FILE * fn_truncate ( const char *, long );

unsigned char fn_getcwd ( char * buffer, unsigned char maxlen, char root );

unsigned char fn_hardformat ( unsigned char fattype );

unsigned char fn_getserial ( unsigned long * );


#if F_FS_THREAD_AWARE == 1

//#include "FreeRTOS/include/FreeRTOS.h"
//#include "FreeRTOS/include/semphr.h"

unsigned char fr_hardformat ( unsigned char fattype );
#define f_hardformat( fattype ) fr_hardformat( fattype )
#define f_format( fattype )    fr_hardformat( fattype )

unsigned char fr_getcwd ( char * buffer, unsigned char maxlen, char root );
#define f_getcwd( buffer, maxlen ) fr_getcwd( buffer, maxlen, 1 )

unsigned char fr_getfreespace ( F_SPACE * pspace );
#define f_getfreespace fr_getfreespace


unsigned char fr_chdir ( const char * dirname );
#define f_chdir( dirname ) fr_chdir( dirname )
unsigned char fr_mkdir ( const char * dirname );
#define f_mkdir( dirname ) fr_mkdir( dirname )
unsigned char fr_rmdir ( const char * dirname );
#define f_rmdir( dirname ) fr_rmdir( dirname )

unsigned char fr_findfirst ( const char * filename, F_FIND * find );
unsigned char fr_findnext ( F_FIND * find );
#define f_findfirst( filename, find ) fr_findfirst( filename, find )
#define f_findnext( find )            fr_findnext( find )

long fr_filelength ( const char * filename );
#define f_filelength( filename ) fr_filelength( filename )

unsigned char fr_close ( F_FILE * filehandle );
F_FILE * fr_open ( const char * filename, const char * mode );

long fr_read ( void * buf, long size, long _size_t, F_FILE * filehandle );

unsigned char fr_getserial ( unsigned long * );
#define f_getserial( serial )  fr_getserial( serial )

unsigned char fr_flush ( F_FILE * f );
#define f_flush( filehandle ) fr_flush( filehandle )

long fr_write ( const void * buf, long size, long _size_t, F_FILE * filehandle );
#define f_write( buf, size, _size_t, filehandle ) fr_write( buf, size, _size_t, filehandle )

unsigned char fr_seek ( F_FILE * filehandle, long offset, unsigned char whence );
#define f_seek( filehandle, offset, whence ) fr_seek( filehandle, offset, whence )

long fr_tell ( F_FILE * filehandle );
#define f_tell( filehandle )     fr_tell( filehandle )
int fr_getc ( F_FILE * filehandle );
#define f_getc( filehandle )     fr_getc( filehandle )
int fr_putc ( int ch, F_FILE * filehandle );
#define f_putc( ch, filehandle ) fr_putc( ch, filehandle )
unsigned char fr_rewind ( F_FILE * filehandle );
#define f_rewind( filehandle )   fr_rewind( filehandle )
unsigned char fr_eof ( F_FILE * filehandle );
#define f_eof( filehandle )      fr_eof( filehandle )

unsigned char fr_delete ( const char * filename );
#define f_delete( filename ) fr_delete( filename )

unsigned char fr_seteof ( F_FILE * );
#define f_seteof( file ) fr_seteof( file )

F_FILE * fr_truncate ( const char *, long );
#define f_truncate( filename, filesize ) fr_truncate( filename, filesize )

#define f_close( filehandle )                    fr_close( filehandle )
#define f_open( filename, mode )                 fr_open( filename, mode )
#define f_read( buf, size, _size_t, filehandle ) fr_read( buf, size, _size_t, filehandle )

#else /* F_FS_THREAD_AWARE */

#define f_hardformat( fattype ) fn_hardformat( fattype )
#define f_format( fattype )    fn_hardformat( fattype )

#define f_getcwd( buffer, maxlen ) fn_getcwd( buffer, maxlen, 1 )

unsigned char fn_getfreespace ( F_SPACE * pspace );
#define f_getfreespace fn_getfreespace


unsigned char fn_chdir ( const char * dirname );
#define f_chdir( dirname ) fn_chdir( dirname )
unsigned char fn_mkdir ( const char * dirname );
#define f_mkdir( dirname ) fn_mkdir( dirname )
unsigned char fn_rmdir ( const char * dirname );
#define f_rmdir( dirname ) fn_rmdir( dirname )

unsigned char fn_findfirst ( const char * filename, F_FIND * find );
unsigned char fn_findnext ( F_FIND * find );
#define f_findfirst( filename, find ) fn_findfirst( filename, find )
#define f_findnext( find )            fn_findnext( find )

#define f_filelength( filename ) fn_filelength( filename )

#define f_getserial( serial )  fn_getserial( serial )

unsigned char fn_flush ( F_FILE * f );
#define f_flush( filehandle ) fn_flush( filehandle )

#define f_write( buf, size, _size_t, filehandle ) fn_write( buf, size, _size_t, filehandle )

#define f_seek( filehandle, offset, whence ) fn_seek( filehandle, offset, whence )

long fn_tell ( F_FILE * filehandle );
#define f_tell( filehandle )     fn_tell( filehandle )
int fn_getc ( F_FILE * filehandle );
#define f_getc( filehandle )     fn_getc( filehandle )
int fn_putc ( int ch, F_FILE * filehandle );
#define f_putc( ch, filehandle ) fn_putc( ch, filehandle )
unsigned char fn_rewind ( F_FILE * filehandle );
#define f_rewind( filehandle )   fn_rewind( filehandle )
unsigned char fn_eof ( F_FILE * filehandle );
#define f_eof( filehandle )      fn_eof( filehandle )

unsigned char fn_delete ( const char * filename );
#define f_delete( filename ) fn_delete( filename )

unsigned char fn_seteof ( F_FILE * );
#define f_seteof( file ) fn_seteof( file )

F_FILE * fn_truncate ( const char *, long );
#define f_truncate( filename, filesize ) fn_truncate( filename, filesize )

#define f_close( filehandle )                    fn_close( filehandle )
#define f_open( filename, mode )                 fn_open( filename, mode )
#define f_read( buf, size, _size_t, filehandle ) fn_read( buf, size, _size_t, filehandle )

#endif /* F_FS_THREAD_AWARE */

/**** Prototype psp_rtc.h *****************************************************/
void psp_getcurrenttimedate ( t_psp_timedate * p_timedate );

/**** Prototype util.h ********************************************************/
void f_igettimedate ( unsigned short * time, unsigned short * date );

unsigned short _f_getword ( void * );
unsigned long _f_getlong ( void * );
char _f_toupper ( char );
void _f_memset ( void *, unsigned char, int );
void _f_memcpy ( void *, void *, int );

void _f_setword ( void *, unsigned short );
void _f_setlong ( void *, unsigned long );
unsigned char * _setcharzero ( int, unsigned char * );
unsigned char * _setchar ( const unsigned char *, int, unsigned char * );
unsigned char * _setword ( unsigned short, unsigned char * );
unsigned char * _setlong ( unsigned long, unsigned char * );

/**** Prototype util_sfn.h ****************************************************/
unsigned char _f_checknamewc ( const char *, const char * );
unsigned char _f_checkname ( char *, char * );

unsigned char _f_setnameext ( char *, char *, char * );
unsigned char _f_setfsname ( const char *, F_NAME * );
int _f_createfullname ( char * buffer, int buffersize, char * path, char * filename, char * fileext );

/**** Prototype volume.h ****************************************************/
extern F_VOLUME  gl_volume;
extern F_FILE    gl_file;
extern char      gl_sector[F_SECTOR_SIZE]; /* actual sector */

unsigned char _f_getvolume ( void );



#endif	/* SDCARD_FAT_SL_H */

