/******************************************************************************/
/* File:   LCD_Keyboard.c                                                     */
/* Author: T McGee                                                            */
/* Created: August 12, 2012                                                */
/******************************************************************************/

#include "SystemConfig.h"

extern UINT8 SV_Index;
extern UINT8 SV_Array[3][10];
// Private Global Variables
UINT8 s;
UINT8 InPutBuff[10];

/******************************************************************************/
/*  Function Name:  KeyBrdScreen()                                            */
/*  Return Value:   void                                                      */
/*  Parameters:     void                                                      */
/*  Description:    Draws the numeric keyboard screen for input.              */
/******************************************************************************/
void KeyBrdScreen(void)
{
    for(s=0;s<10;s++)
        {
            InPutBuff[s] = '\0';
        }
    
    setBackColor(251,200,180);                      // Set color for main fill of keyboard
    fillRoundRect (170, 50, 635, 430);              // Draw main fill
    setForColor(0, 0, 0);                           // Set color for keyboard main frame
    drawRoundRect (170, 50, 635, 430, 3);           // Draw main frame
        
    // Top row
    //Input screen at end of function to leave backround color setting
    setBackColor(209, 211, 212);                    // Set color for button faces
    fillRoundRect (545, 335, 620, 410);             // Draw backspace buton
    drawRoundRect (545, 335, 620, 410, 3);
    printString("<<", 565, 362);


    // Draw the second row of buttons
    for (s=0; s<5; s++)                             // Draw 1-5 buttons
    {                                   
      fillRoundRect (185+(s*90), 245, 260+(s*90), 320);
      drawRoundRect (185+(s*90), 245, 260+(s*90), 320, 3);
      printChar(s+49, 214+(s*90), 273);
    }

    // Draw the third row of buttons
    for (s=0; s<5; s++)                             // Draw 5-9 plus zero buttons
    {                            
      fillRoundRect (185+(s*90), 155, 260+(s*90), 230);
      drawRoundRect (185+(s*90), 155, 260+(s*90), 230, 3);
      if (s<4) printChar(s+54, 214+(s*90), 183);
    }
    printChar(48, 574, 183);                        // Print zero

    // Draw the forth row of buttons
                           
    fillRoundRect (185, 65, 260, 140);              // Draw decimal point button
    drawRoundRect (185, 65, 260, 140, 3);
    printString(".", 215, 95);
                                  
    fillRoundRect (275, 65, 440, 140);              // Draw Cancel button
    drawRoundRect (275, 65, 440, 140, 3);  
    printString("Cancel",310, 95);
                                
    fillRoundRect (455, 65, 620, 140);              // Draw Enter button
    drawRoundRect (455, 65, 620, 140, 3);    
    printString("Enter", 495, 95);

    //Input screen
    setBackColor(157,210,155);                      // Set color for input screen
    fillRoundRect (185, 335, 530, 410);             // Draw input screen
    drawRoundRect (185, 335, 530, 410, 3);

    s=0;

}

/******************************************************************************/
/*  Function Name:  KeyBrdResponse                                            */
/*  Return Value:   void                                                      */
/*  Parameters:     void                                                      */
/*  Description:    Finds button touched by user and response as appropriately*/
/******************************************************************************/
void KeyBrdResponse(void)
{   // Maximum digits in InPutBuff is set for 10
    
    // First row
    if ((TP_Y>335) && (TP_Y<410))                         // BackSpace button
    {
        if ((TP_X>545) && (TP_X<620))
        {
            if(ServiceRequests.TS_DIR)
            {
                KeyBrdDeselect(335, 545, 410, 620,3);
                if (s)
                {
                    InPutBuff[--s] = '\0';
                    setBackColor(251,200,180);
                    printString("          ", 350, 362);
                    printString(InPutBuff, (500-s*16), 362);
                }
            }
            else
            {
                KeyBrdSelect(335, 545, 410, 620,3);
            }
        }
    }

    // Second Row
    else if ((TP_Y>245) && (TP_Y<320))
      {
        if ((TP_X>185) && (TP_X<260))                     // Button: 1
        {
            if(ServiceRequests.TS_DIR)
            {
                KeyBrdDeselect(185, 245, 260, 320,3);
                if(s<9)InPutBuff[s++] = '1';
                setBackColor(251,200,180);
                printString(InPutBuff, (500-s*16), 362);
            }
            else
            {
                KeyBrdSelect(185, 245, 260, 320,3);
            }
        }
        else if ((TP_X>275) && (TP_X<350))                // Button: 2
        {
            if(ServiceRequests.TS_DIR)
            {
                KeyBrdDeselect(275, 245, 350, 320,3);
                if(s<9)InPutBuff[s++] = '2';
                setBackColor(251,200,180);
                printString(InPutBuff, (500-s*16), 362);
            }
            else
            {
                KeyBrdSelect(275, 245, 350, 320,3);
            }
        }
        else if ((TP_X>365) && (TP_X<440))                // Button: 3
        {
            if(ServiceRequests.TS_DIR)
            {
                KeyBrdDeselect(365, 245, 440, 320,3);
                if(s<9)InPutBuff[s++] = '3';
                setBackColor(251,200,180);
                printString(InPutBuff, (500-s*16), 362);
            }
            else
            {
                KeyBrdSelect(365, 245, 440, 320,3);
            }

        }
        else if ((TP_X>465) && (TP_X<530))                // Button: 4
        {
            if(ServiceRequests.TS_DIR)
            {
                KeyBrdDeselect(455, 245, 530, 320,3);
                if(s<9)InPutBuff[s++] = '4';
                setBackColor(251,200,180);
                printString(InPutBuff, (500-s*16), 362);
            }
            else
            {
                KeyBrdSelect(455, 245, 530, 320,3);
            }
        }
        else if ((TP_X>545) && (TP_X<=620))               // Button: 5
        {
            if(ServiceRequests.TS_DIR)
            {
                KeyBrdDeselect(545, 245, 620, 320,3);
                if(s<9)InPutBuff[s++] = '5';
                setBackColor(251,200,180);
                printString(InPutBuff, (500-s*16), 362);
            }
            else
            {
                KeyBrdSelect(545, 245, 620, 320,3);
            }
        }
      }

    //Third row
    else if ((TP_Y>155) && (TP_Y<230))
      {
        if ((TP_X>185) && (TP_X<260))                     // Button: 6
        {
            if(ServiceRequests.TS_DIR)
            {
                KeyBrdDeselect(185, 155, 260, 230,3);
                if(s<9)InPutBuff[s++] = '6';
                setBackColor(251,200,180);
                printString(InPutBuff, (500-s*16), 362);
            }
            else
            {
                KeyBrdSelect(185, 155, 260, 230,3);
            }
        }
        else if ((TP_X>275) && (TP_X<350))                // Button: 7
        {
            if(ServiceRequests.TS_DIR)
            {
                KeyBrdDeselect(275, 155, 350, 230,3);
                if(s<9)InPutBuff[s++] = '7';
                setBackColor(251,200,180);
                printString(InPutBuff, (500-s*16), 362);
            }
            else
            {
                KeyBrdSelect(275, 155, 350, 230,3);
            }
        }
        else if ((TP_X>365) && (TP_X<440))                // Button: 8
        {
            if(ServiceRequests.TS_DIR)
            {
                KeyBrdDeselect(365, 155, 440, 230,3);
                if(s<9)InPutBuff[s++] = '8';
                setBackColor(251,200,180);
                printString(InPutBuff, (500-s*16), 362);
            }
            else
            {
                KeyBrdSelect(365, 155, 440, 230,3);
            }
        }
        else if ((TP_X>465) && (TP_X<530))                // Button: 9
        {
            if(ServiceRequests.TS_DIR)
            {
                KeyBrdDeselect(455, 155, 530, 230,3);
                if(s<9)InPutBuff[s++] = '9';
                setBackColor(251,200,180);
                printString(InPutBuff, (500-s*16), 362);
            }
            else
            {
                KeyBrdSelect(455, 155, 530, 230,3);
            }
        }
        else if ((TP_X>545) && (TP_X<=620))               // Button: 0
        {
            if(ServiceRequests.TS_DIR)
            {
                KeyBrdDeselect(545, 155, 620, 230,3);
                if(s<9)InPutBuff[s++] = '0';
                setBackColor(251,200,180);
                printString(InPutBuff, (500-s*16), 362);
            }
            else
            {
                KeyBrdSelect(545, 155, 620, 230,3);
            }
        }
      }

    // Fourth row
    else if ((TP_Y>65) && (TP_Y<140))                     // Bottom row
      {
        if ((TP_X>185) && (TP_X<260))                     // Button: Decimal point
        {
            if(ServiceRequests.TS_DIR)
            {
                KeyBrdDeselect(185, 65, 260, 140,3);
                if(s<9)InPutBuff[s++] = '.';
                setBackColor(251,200,180);
                printString(InPutBuff, (500-s*16), 362);
            }
            else
            {
                KeyBrdSelect(185, 65, 260, 140,3);
            }
        }
        else if ((TP_X>275) && (TP_X<440))                // Button: Cancel
        {
            if(ServiceRequests.TS_DIR)
            {
                KeyBrdDeselect(275, 65, 440, 140,3);
                --ScreenIndex;
                MainScreen();                            // Keyboard should now return to previous screen
            }
            else
            {
                KeyBrdSelect(275, 65, 440, 140,3);
            }
        }
        else if ((TP_X>455) && (TP_X<620))                // Button: Enter
        {
            if(ServiceRequests.TS_DIR)
            {
                KeyBrdDeselect(455, 65, 620, 140,3);
                for(s=0;s<10;s++)
                {
                    SV_Array[SV_Index][s] = InPutBuff[s];
                }
                --ScreenIndex;
                MainScreen();                            // Keyboard should now return to previous screen
            }
            else
            {
                KeyBrdSelect(455, 65, 620, 140,3);
            }
        }
      }
}

/******************************************************************************/
/*  Function Name:  KeyBrdSelection()                                         */
/*  Return Value:   void                                                      */
/*  Parameters:     UINT16 X and Y corners of button frame to highlight.      */
/*  Description:    Highlights selection with a red frame while a button      */
/*                  is touched. waits for release and debounces button.       */
/******************************************************************************/
void KeyBrdSelect(UINT16 x1, UINT16 y1, UINT16 x2, UINT16 y2, UINT8 w )
{
    // Highlight selection with a red frame while a button is touched
    setForColor(255, 0, 0);
    drawRoundRect (x1, y1, x2, y2, w);
    setForColor(75, 75, 75);                    //Reset Forground color to black

    ServiceRequests.TS_DIR = 1;
    ServiceRequests.TS_DWN = 0;
    ConfigINT3(EXT_INT_PRI_3 | RISING_EDGE_INT | EXT_INT_ENABLE);
}

void KeyBrdDeselect(UINT16 x1, UINT16 y1, UINT16 x2, UINT16 y2, UINT8 w )
{
    unsigned long t;

    t = TickGet(0);                             //Zero timer
    do{
       if (! TS_IRQ) t = TickGet(0);            //Wait for touch screen release
    }while((TickGet(0) - t < TS_DB_EXIT));      //Debounce touch screen then exit.

    drawRoundRect (x1, y1, x2, y2, w);          //Clear red frame

    ServiceRequests.TS_DIR = 0;
    ServiceRequests.TS_UP = 0;
    ConfigINT3(EXT_INT_PRI_3 | FALLING_EDGE_INT | EXT_INT_ENABLE);

}