/*******************************************************************************
 * File:   50_LCD.c
 * Author: T McGee
 * Created: July 16, 2012
 ******************************************************************************/
#include "SystemConfig.h"
#include <math.h>           //For sin and cos functions

/*******************************************************************************
 * User Type Definitions
 ******************************************************************************/

    typedef struct              //Structure for bitmap data
    {
        union{
            long size_L;
            struct {char size_0; char size_1; char size_2; char size_3; };
            };
        union{
            long offset_L;
            struct {char offset_0; char offset_1; char offset_2; char offset_3; };
            };
        union{
            long x_size_L;
            struct {int x_size_int0; int x_size_int1; };
            struct {char x_size_0; char x_size_1; char x_size_2; char x_size_3; };
            };
        union{
            long y_size_L;
            struct {int y_size_int0; int y_size_int1; };
            struct {char y_size_0; char y_size_1; char y_size_2; char y_size_3; };
            };
        union{
            int compress_I;
            struct {char compress_L; char compress_H; };
            };
        union{
            int depth_I;
            struct {char depth_L; char depth_H; };
            };
    }bmp_header;

/*******************************************************************************
 *  Function Name:  void LCD_Init(void)
 *  Return Value:   void
 *  Parameters:     void
 *  Description:    Initializes IO Pins, LCD bus, LCD panel, SPI touch bus.
 ******************************************************************************/
void LCD_Init()
{
    //Init LCD screen
    SEL_RST;                        // #1 Hardware Reset
    SEL_GP;                         // Pull low to reset.
    delayMS(5);;                  // Delay 5mS

    DES_RST;
    DESELECT();
    delayMS(10);                 // Delay 10mS

    SEL_CM;
    SEL_GP;
    comWritPort(0x00,0xE2);         // #2 Set up PLL
    DESELECT();
    SEL_DT;
    SEL_GP;
    comWritPort(0x00,0x1D);         // N=0x1D (29)
    comWritPort(0x00,0x02);         // M=0x02  (2)  12MHz*30/3=120MHz
    comWritPort(0x00,0x54);         // Dummy Byte
    DESELECT();

    SEL_CM;
    SEL_GP;
    comWritPort(0x00,0xE0);         // #3 Turn on PLL
    DESELECT();
    SEL_DT;
    SEL_GP;
    comWritPort(0x00,0x01);
    DESELECT();
    delayMS(1);                  // #4 Let PLL stablize

    SEL_CM;
    SEL_GP;
    comWritPort(0x00,0xE0);         // #5 Switch Clock to PLL
    DESELECT();
    SEL_DT;
    SEL_GP;
    comWritPort(0x00,0x03);
    DESELECT();
    delayMS(1);                  // Delay 1mS

    SEL_CM;
    SEL_GP;
    comWritPort(0x00,0x01);         // #6 software reset
    DESELECT();
    delayMS(100);                // Delay 100mS

    SEL_CM;
    SEL_GP;
    comWritPort(0x00,0xE6);         // #7 Configure Dot CLK
    DESELECT();
    SEL_DT;
    SEL_GP;
    comWritPort(0x00,0x03);         // C_FPR = 262143
    comWritPort(0x00,0xFF);
    comWritPort(0x00,0xFF);
    DESELECT();

    SEL_CM;
    SEL_GP;
    comWritPort(0x00,0xB0);         // #8A LCD SPECIFICATION
    DESELECT();
    SEL_DT;
    SEL_GP;
    comWritPort(0x00,0x24);         // Para1 0b00100100

    comWritPort(0x00,0x00);         // Para2 0B00000000

    comWritPort(0x00,0x03);         // Para3 Set HPS 799
    comWritPort(0x00,0x1F);         // Para4

    comWritPort(0x00,0x01);         // Para5 Set VPS 479
    comWritPort(0x00,0xDF);         // Para6

    comWritPort(0x00,0x00);         // Para7 Even, odd RGB sequence
    DESELECT();

    SEL_CM;
    SEL_GP;
    comWritPort(0x00,0xB4);         // #8B HSYNC
    DESELECT();
    SEL_DT;
    SEL_GP;
    comWritPort(0x00,0x03);         // Para1 Set HT 927
    comWritPort(0x00,0x9F);         // Para2

    comWritPort(0x00,0x00);         // Para3 Set HPS 88
    comWritPort(0x00,0x58);         // Para4

    comWritPort(0x00,0x2F);         // Para5 Set HPW 47

    comWritPort(0x00,0x00);         // Para6 Set LPS 0
    comWritPort(0x00,0x00);         // Para7

    comWritPort(0x00,0x00);         // Para8 Set LPSPP 0
    DESELECT();

    SEL_CM;
    SEL_GP;
    comWritPort(0x00,0xB6);         // #8C VSYNC
    DESELECT();
    SEL_DT;
    SEL_GP;
    comWritPort(0x00,0x02);         // Para1 Set VT	524
    comWritPort(0x00,0x0C);         // Para2

    comWritPort(0x00,0x00);         // Para3 Set VPS	32
    comWritPort(0x00,0x20);         // Para4

    comWritPort(0x00,0x02);         // Para5 Set VPW	2

    comWritPort(0x00,0x00);         // Para6 Set FPS	0
    comWritPort(0x00,0x00);         // Para7
    DESELECT();

    SEL_CM;
    SEL_GP;
    comWritPort(0x00,0xBE);         // #9 PWM configuration
    DESELECT();
    SEL_DT;
    SEL_GP;
    comWritPort(0x00,0x05);         // PWM N
    comWritPort(0x00,0xFF);         // PWM dyty cycle (50% = 0x80)
    comWritPort(0x00,0x01);         // Disable DBC
    DESELECT();
                                    // #10 Configur GPIO (NA)

    SEL_CM;
    SEL_GP;
    comWritPort(0x00,0x29);         // #11 display on
    DESELECT();

    setXY(0, 0, 799, 479);          // #12 Frame Buffer Configuration

    SEL_CM;
    SEL_GP;
    comWritPort(0x00,0x36);         // #13 Configure rotation
    DESELECT();
    SEL_DT;
    SEL_GP;
    comWritPort(0x00,0b00000001);   //
    DESELECT();

    SEL_CM;
    SEL_GP;
    comWritPort(0x00,0xF0);         // #14 MUC Bus Type
    DESELECT();
    SEL_DT;
    SEL_GP;
    comWritPort(0x00,0x03);         // 0b00000011, 16 bit, 565 formate
    DESELECT();

}

void setFont(char* font)
{
    cfont.font=font;
    
    cfont.x_size   = cfont.font[0];
    cfont.y_size   = cfont.font[1];
    cfont.offset   = cfont.font[2];
    cfont.numchars = cfont.font[3];
}

void setForColor(char r, char g, char b)
{
    ForColorH=((r&248)|g>>5);
    ForColorL=((g&28)<<3|b>>3);
}

void setBackColor(char r, char g, char b)
{
    BackColorH=((r&248)|g>>5);
    BackColorL=((g&28)<<3|b>>3);
}

void clrScr()
{
    long i;

    setXY(0,0,799,479);

    SEL_DT;
    for (i=0; i<384000; i++)
    {
        SEL_GP;                       // Enable Data transfer
        comWritPort(0,0);
        DESELECT();                       // Disable transfer
    }

}

void setXY(int x1, int y1, int x2, int y2)
{
    bytelong temp;
    
    SEL_CM;
    SEL_GP;                         
    comWritPort(0x00,0x2A);     // Set column address
    DESELECT();
    temp.int0 = x1;
    SEL_DT;
    SEL_GP;                          
    comWritPort(0x00,temp.b1);
    comWritPort(0x00,temp.b0);
    temp.int0 = x2;
    comWritPort(0x00,temp.b1);
    comWritPort(0x00,temp.b0);
    DESELECT();

    SEL_CM;
    SEL_GP;                         
    comWritPort(0x00,0x2B);     // Set page address
    DESELECT();
    temp.int0 = y1;
    SEL_DT;
    SEL_GP;                          
    comWritPort(0x00,temp.b1);
    comWritPort(0x00,temp.b0);
    temp.int0 = y2;
    comWritPort(0x00,temp.b1);
    comWritPort(0x00,temp.b0);
    DESELECT();
    
    SEL_CM;
    SEL_GP;                         
    comWritPort(0x00,0x2C);    // Write memory start
    DESELECT();
 
}

void fillScr(char r, char g, char b)
{
    long i;
    char ch, cl;

    ch=((r&248)|g>>5);
    cl=((g&28)<<3|b>>3);
    setXY(0,0,799,479);

    SEL_DT;                         // Enable data transfer
    for (i=0; i<384000; i++)
    {
        SEL_GP;                     // Enable transfer
        comWritPort(ch,cl);
        DESELECT();                   // Disable transfer
    }
 }

void drawRect(int x1, int y1, int x2, int y2)
{
    drawHLine(x1, y1, x2-x1);
    drawHLine(x1, y2, x2-x1);
    drawVLine(x1, y1, y2-y1);
    drawVLine(x2, y1, y2-y1);
}

void fillRect(int x1, int y1, int x2, int y2)
{
    int i;


    for (i=0; i<((y2-y1)/2)+1; i++)
    {
        drawFillBar(x1, y1+i, x2-x1);
        drawFillBar(x1, y2-i, x2-x1);
    }

}

void drawRoundRect(int x1, int y1, int x2, int y2, char w)
{
    char i;
    
    for(i=0;i<w;i++)
    {
        drawPixel(x1+1+i,y1+1+i);
        drawPixel(x2-1-i,y1+1+i);
        drawPixel(x1+1+i,y2-1-i);
        drawPixel(x2-1-i,y2-1-i);
        drawHLine(x1+2, y1+i, x2-x1-4);
        drawHLine(x1+2, y2-i, x2-x1-4);
        drawVLine(x1+i, y1+2, y2-y1-4);
        drawVLine(x2-i, y1+2, y2-y1-4);       
    }
}

void fillRoundRect(int x1, int y1, int x2, int y2)
{
    int i,l;
    l = x2-x1;

    for (i=0; i<((y2-y1)/2)+1; i++)
    {
        switch(i)
        {
            case 0:
                drawFillBar(x1+2, y1, l-4);
                drawFillBar(x1+2, y2, l-4);
                break;
            case 1:
                drawFillBar(x1+1, y1+1, l-2);
                drawFillBar(x1+1, y2-1, l-2);
                break;
            default:
                drawFillBar(x1, y1+i, l);
                drawFillBar(x1, y2-i, l);
        }
    }
}

void drawCircle2(int x, int y, int r, char w)
{
    char i,f;
    int j,k=0;
    
    for(i=0;i<w;i++)
    {
        while(k <= j)
        {
            for(f=0;f<2;f++)
            {
                if(f==0) j=(int)ceil(sqrt(((double)r*r-k*k)));
                else j=(int)floor(sqrt(((double)r*r-k*k)));
                
                setXY(x-j,y+k,x-j,y+k);
                SEL_DT;
                SEL_GP;                          // Enable Data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();
                setXY(x-j,y-k,x-j,y-k);
                SEL_DT;
                SEL_GP;                          // Enable Data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();
                setXY(x+j,y+k,x+j,y+k);
                SEL_DT;
                SEL_GP;                          // Enable Data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();
                setXY(x+j,y-k,x+j,y-k);
                SEL_DT;
                SEL_GP;                          // Enable Data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();
                setXY(x-k,y+j,x-k,y+j);
                SEL_DT;
                SEL_GP;                       // Enable Data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();
                setXY(x-k,y-j,x-k,y-j);
                SEL_DT;
                SEL_GP;                          // Enable Data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();
                setXY(x+k,y+j,x+k,y+j);
                SEL_DT;
                SEL_GP;                          // Enable Data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();
                setXY(x+k,y-j,x+k,y-j);
                SEL_DT;
                SEL_GP;                          // Enable Data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();
            }
            k++;
        }
        r--;
        k=0;
    }
}

void draw1stQuad(int x, int y, int r, char w)
{
    char i,f;
    int j,k=0,w_2, w_f;


    w_2 = w/2;
    r = r - w_2;
    w_f = w_2 + w_2 + 1;

    for(i=0;i<w;i++)
    {
        while(k <= j)
        {
            for(f=0;f<2;f++)
            {
                if(f==0) j=(int)ceil(sqrt(((double)r*r-k*k)));
                else j=(int)floor(sqrt(((double)r*r-k*k)));

                setXY(x+j,y+k,x+j,y+k);
                SEL_DT;
                SEL_GP;                          // Enable Data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();
                setXY(x+k,y+j,x+k,y+j);
                SEL_DT;
                SEL_GP;                          // Enable Data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();
            }
            k++;
        }
        r++;
        k=0;
    }
}

void draw2ndQuad(int x, int y, int r, char w)
{
    char i,f;
    int j,k=0,w_2, w_f;
     

    w_2 = w/2;
    r = r - w_2;
    w_f = w_2 + w_2 + 1;

    for(i=0;i<w;i++)
    {
        while(k <= j)
        {
            for(f=0;f<2;f++)
            {
                if(f==0) j=(int)ceil(sqrt(((double)r*r-k*k)));
                else j=(int)floor(sqrt(((double)r*r-k*k)));

                setXY(x-j,y+k,x-j,y+k);
                SEL_DT;
                SEL_GP;                          // Enable Data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();
                setXY(x-k,y+j,x-k,y+j);
                SEL_DT;
                SEL_GP;                          // Enable Data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();
            }
            k++;
        }
        r++;
        k=0;
    }
}

void draw3rdQuad(int x, int y, int r, char w)
{
    char i,f;
    int j,k=0,w_2, w_f;


    w_2 = w/2;
    r = r - w_2;
    w_f = w_2 + w_2 + 1;

    for(i=0;i<w;i++)
    {
        while(k <= j)
        {
            for(f=0;f<2;f++)
            {
                if(f==0) j=(int)ceil(sqrt(((double)r*r-k*k)));
                else j=(int)floor(sqrt(((double)r*r-k*k)));

                setXY(x-j,y-k,x-j,y-k);
                SEL_DT;
                SEL_GP;                          // Enable Data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();
                setXY(x-k,y-j,x-k,y-j);
                SEL_DT;
                SEL_GP;                          // Enable Data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();
            }
            k++;
        }
        r++;
        k=0;
    }
}

void drawCircle(int x, int y, int radius, char w)
{
    char i;
    for(i=0;i<w;i++)
    {
	int f = 1 - radius;
	int ddF_x = 1;
	int ddF_y = -2 * radius;
	int x1 = 0;
	int y1 = radius;
        
	setXY(x, y + radius, x, y + radius);
        SEL_DT;
        SEL_GP;                          // Enable data transfer
        comWritPort(ForColorH,ForColorL);
        DESELECT();                          // Disable transfer
	setXY(x, y - radius, x, y - radius);
        SEL_DT;
        SEL_GP;                          // Enable data transfer
        comWritPort(ForColorH,ForColorL);
        DESELECT();                          // Disable transfer
	setXY(x + radius, y, x + radius, y);
        SEL_DT;
        SEL_GP;                          // Enable data transfer
        comWritPort(ForColorH,ForColorL);
        DESELECT();                          // Disable transfer
	setXY(x - radius, y, x - radius, y);
        SEL_DT;
        SEL_GP;                          // Enable data transfer
        comWritPort(ForColorH,ForColorL);
        DESELECT();                          // Disable transfer

        while(x1 < y1)
	{
            if(f >= 0)
            {
		y1--;
		ddF_y += 2;
		f += ddF_y;
            }
            x1++;
            ddF_x += 2;
            f += ddF_x;
            setXY(x + x1, y + y1, x + x1, y + y1);
            SEL_DT;
            SEL_GP;                          // Enable data transfer
            comWritPort(ForColorH,ForColorL);
            DESELECT();                          // Disable transfer
            setXY(x - x1, y + y1, x - x1, y + y1);
            SEL_DT;
            SEL_GP;                          // Enable data transfer
            comWritPort(ForColorH,ForColorL);
            DESELECT();                          // Disable transfer
            setXY(x + x1, y - y1, x + x1, y - y1);
            SEL_DT;
            SEL_GP;                          // Enable data transfer
            comWritPort(ForColorH,ForColorL);
            DESELECT();                          // Disable transfer
            setXY(x - x1, y - y1, x - x1, y - y1);
            SEL_DT;
            SEL_GP;                          // Enable data transfer
            comWritPort(ForColorH,ForColorL);
            DESELECT();                          // Disable transfer
            setXY(x + y1, y + x1, x + y1, y + x1);
            SEL_DT;
            SEL_GP;                          // Enable data transfer
            comWritPort(ForColorH,ForColorL);
            DESELECT();                          // Disable transfer
            setXY(x - y1, y + x1, x - y1, y + x1);
            SEL_DT;
            SEL_GP;                          // Enable data transfer
            comWritPort(ForColorH,ForColorL);
            DESELECT();                          // Disable transfer
            setXY(x + y1, y - x1, x + y1, y - x1);
            SEL_DT;
            SEL_GP;                          // Enable data transfer
            comWritPort(ForColorH,ForColorL);
            DESELECT();                          // Disable transfer
            setXY(x - y1, y - x1, x - y1, y - x1);
            SEL_DT;
            SEL_GP;                          // Enable data transfer
            comWritPort(ForColorH,ForColorL);
            DESELECT();                          // Disable transfer
	}
        radius--;
    }
}

void fillCircle(int x, int y, int radius)
{
    int y1,x1 ;
   
    for(y1=-radius; y1<=radius; y1++)
        for(x1=-radius; x1<=radius; x1++)
            if(x1*x1+y1*y1 <= radius*radius)
            {
                setXY(x+x1, y+y1, x+x1, y+y1);
                SEL_DT;
                SEL_GP;                          // Enable data transfer
                comWritPort(BackColorH,BackColorL);
                DESELECT();                          // Disable transfer
            }
}

void drawFillBar(int x, int y, int l)
{
    int i;

    setXY(x, y, x+l, y);
    SEL_DT;
    for (i=0; i<l+1; i++)
    {     
        SEL_GP;                          // Enable data transfer
        comWritPort(BackColorH,BackColorL);
        DESELECT();                          // Disable transfer
    }
        
}

void drawHLine(int x, int y, int l)
{
    int i;

    setXY(x, y, x+l, y);
    SEL_DT;
    for (i=0; i<l+1; i++)
    {
        SEL_GP;                          // Enable data transfer
        comWritPort(ForColorH,ForColorL);
        DESELECT();                          // Disable transfer
    }

}

void drawHLWide(int x, int y, int l, int w)
{
        int i, k, w_2, w_f;

        w_2 = w/2;
        y = y - w_2;
        w_f = w_2 + w_2 + 1;
                       
        for(k=0;k<w_f;k++)
        {
            setXY(x, y+k, x+l, y+k);
            SEL_DT;
            for (i=0; i<l+1; i++)
            {
                SEL_GP;                      // Enable data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();                      // Disable transfer
            }
        }
                              	
}

void drawArrowL(int x, int y, int w)
{
    int i,j,f,w_2,x1;

        w_2=w/2;
        f=w=w_2*2+1;
        x1=x-w;
       
        for(i=0;i<w;i++)
        {
            setXY(x1+i,y+i,x,y+i);
            SEL_DT;
            for(j=0;j<f;j++)
            {
                SEL_GP;                      // Enable data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();                      // Disable transfer
            }
            setXY(x1+i,y-i,x,y-i);
            SEL_DT;
            for(j=0;j<f;j++)
            {
                SEL_GP;                      // Enable data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();                      // Disable transfer
            }
            f--;
        }

}

void drawArrowR(int x, int y, int w)
{
    int i,j,f,w_2,x2;

        w_2=w/2;
        f=w=w_2*2+1;
        x2=x+w;

        for(i=0;i<w;i++)
        {
            setXY(x,y+i,x2-i,y+i);
            SEL_DT;
            for(j=0;j<f;j++)
            {
                SEL_GP;                      // Enable data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();                      // Disable transfer
            }
            setXY(x,y-i,x2-i,y-i);
            SEL_DT;
            for(j=0;j<f;j++)
            {
                SEL_GP;                      // Enable data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();                      // Disable transfer
            }
            f--;
        }

}

void drawVLine(int x, int y, int l)
{
    int i;

    setXY(x, y, x, y+l);
    SEL_DT;
    for (i=0; i<l; i++)
    {
	SEL_GP;                          // Enable data transfer
        comWritPort(ForColorH,ForColorL);
        DESELECT();                          // Disable transfer
    }
  
}

void drawVLWide(int x, int y, int l, int w)
{
        int i, k, w_2, w_f;

        w_2 = w/2;
        x = x - w_2;
        w_f = w_2 + w_2 + 1;
                      
        for(k=0;k<w_f;k++)
        {
            setXY(x+k, y, x+k, y+l);
            SEL_DT;
            for (i=0; i<l; i++)
            {
                SEL_GP;                  // Enable data transfer
		comWritPort(ForColorH,ForColorL);
                DESELECT();                  // Disable transfer
            }
        }                          
	
}

void drawLine(int x1, int y1, int x2, int y2)
{
    double delta, tx, ty;
    int i;

    if (abs(x2-x1)>abs(y2-y1))
    {
        delta=(double)(y2-y1)/(double)(x2-x1);
        ty=(double)(y1);

        for (i=x1; i<=x2; i++)
        {
            setXY(i, (int)ty, i, (int)ty);
            SEL_DT;
            SEL_GP;                          // Enable data transfer
            comWritPort(ForColorH,ForColorL);
            DESELECT();                          // Disable transfer
            ty=ty+delta;
        }
    }

    else
    {

        delta=((double)(x2-x1)/(double)(y2-y1));
        tx=(double)(x1);

        if (y1>y2)
        {
            delta = -delta;
            for (i=y1; i>y2-1; i--)
            {
                setXY((int)tx, i, (int)tx, i);
                SEL_DT;
		SEL_GP;                          // Enable data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();                          // Disable transfer
        	tx=tx+delta;
            }
        }
        else
        {
            for (i=y1; i<y2+1; i++)
            {
                setXY((int)tx, i, (int)tx, i);
                SEL_DT;
		SEL_GP;                          // Enable data transfer
                comWritPort(ForColorH,ForColorL);
                DESELECT();                          // Disable transfer
        	tx=tx+delta;
            }
        }
    }
}

void drawPixel(int x, int y)
{
    setXY(x, y, x, y);
    SEL_DT;
    SEL_GP;                          // Enable data transfer
    comWritPort(ForColorH,ForColorL);
    DESELECT();                          // Disable transfer
   
}

void setPixel(char r, char g, char b)
{
    SEL_DT;
    SEL_GP;                          // Enable data transfer
    comWritPort(((r&248)|g>>5),((g&28)<<3|b>>3));
    DESELECT();                          // Disable transfer
}

void printChar(char c, int x, int y)
{
    char i, ix, fIndex, ch;
    unsigned int j,ch_pt;

    y += cfont.y_size;
    fIndex = cfont.x_size/8;
    ch_pt = (c-cfont.offset)*(fIndex*cfont.y_size)+4;

    for(j=0;j<cfont.y_size;j++)
    {
        setXY(x,y-j,x+cfont.x_size-1,y-j);
        SEL_DT;
        for(ix=0;ix<fIndex;ix++)
        {
            ch=cfont.font[ch_pt++];
            for(i=8;i>0;i--)
            {
                if((ch&(1<<(i-1)))!=0) 
                {
                    SEL_GP;                          // Enable data transfer
                    comWritPort(ForColorH,ForColorL);
                    DESELECT();                          // Disable transfer
                }
                else 
                {
                    SEL_GP;                          // Enable data transfer
                    comWritPort(BackColorH,BackColorL);
                    DESELECT();                        // Disable transfer
                }
            }
        }
    }
}

void printString(char *st, int x, int y)
{
    int stl, i, x2=x;

    stl = strlen(st);
    for(i=0;i<stl;i++)
    {
        if(st[i] == '\r') x = x2;
        else if(st[i] == '\n') (y -= (cfont.y_size + 5));
        else
        {
            printChar(st[i], x, y);
            x += cfont.x_size;
        }
    }
}

void drawBitmap(int x, int y, const unsigned char* image)
{
    char r,g,b;
    int ix, iy, x2;
    long s;
    bmp_header bmp;

    bmp.size_0 = image[2];        // Set bmp Size
    bmp.size_1 = image[3];
    bmp.size_2 = image[4];
    bmp.size_3 = image[5];

    bmp.offset_0 = image[10];     // Set offset
    bmp.offset_1 = image[11];
    bmp.offset_2 = image[12];
    bmp.offset_3 = image[13];

    bmp.x_size_0 = image[18];     // Set X dimension
    bmp.x_size_1 = image[19];
    bmp.x_size_2 = image[20];
    bmp.x_size_3 = image[21];

    bmp.y_size_0 = image[22];     // Set Y dimension
    bmp.y_size_1 = image[23];
    bmp.y_size_2 = image[24];
    bmp.y_size_3 = image[25];

    bmp.compress_L = image[26];   // Compress
    bmp.compress_H = image[27];


    bmp.depth_L = image[28];      // Color depth
    bmp.depth_H = image[29];
 

    s = bmp.offset_L;
    x2 = x + bmp.x_size_int0;

    for (iy=0; iy<bmp.y_size_int0; iy++)
    {
        setXY(x, y+iy, x2, y+iy);
        SEL_DT;
        for (ix = 0; ix<bmp.x_size_int0; ix++)
        {
            r = image[s++];
            g = image[s++];
            b = image[s++];
            SEL_GP;       // Enable data transfer
            comWritPort(((b&248)|g>>5),((g&28)<<3|r>>3));
            DESELECT();
        }
    }

}
/*
#ifdef SD_CARD
void drawBitmapFile(int x, int y, char* FileName)
{
    char image[33];
    int ix, iy, x2, LineLength;
    long s;
    bmp_header bmp;    
    FSFILE* ptImage; 
    
    ptImage = FSfopen (FileName, "r");
    FSfread (image, 32, 1, ptImage);

    bmp.size_0 = image[2];        // Set bmp Size
    bmp.size_1 = image[3];
    bmp.size_2 = image[4];
    bmp.size_3 = image[5];

    bmp.offset_0 = image[10];     // Set offset
    bmp.offset_1 = image[11];
    bmp.offset_2 = image[12];
    bmp.offset_3 = image[13];

    bmp.x_size_0 = image[18];     // Set X dimension
    bmp.x_size_1 = image[19];
    bmp.x_size_2 = image[20];
    bmp.x_size_3 = image[21];

    bmp.y_size_0 = image[22];     // Set Y dimension
    bmp.y_size_1 = image[23];
    bmp.y_size_2 = image[24];
    bmp.y_size_3 = image[25];

    bmp.compress_L = image[26];   // Compress
    bmp.compress_H = image[27];

    bmp.depth_L = image[28];      // Color depth
    bmp.depth_H = image[29];

    s = bmp.offset_L;
    FSfseek(ptImage, s, SEEK_SET);
    LineLength = bmp.x_size_int0 * 3;    
    x2 = x + bmp.x_size_int0;

    for (iy = 0; iy < bmp.y_size_int0; iy++)
    {
        FSfread (ImageLine, LineLength, 1, ptImage);
        setXY(x, y+iy, x2, y+iy);
        SEL_DT;
        for (ix=0; ix < LineLength; ix += 3)
        {       
            SEL_GP;
            comWritPort(((ImageLine[(ix+2)]&248)|ImageLine[(ix+1)]>>5),((ImageLine[(ix+1)]&28)<<3|ImageLine[ix]>>3));
            DESELECT();
        }
    }
    FSfclose (ptImage);
}
#endif
*/

#ifdef FLASH_MEMORY
void drawBitmapFlash(int x, int y, long imageAdres)
{
    char imageHeader[33];
    int ix, iy, x2, LineLength;
    bmp_header bmp;

    ReadAddr.Long = imageAdres;
    SPIFlashReadArray(imageHeader, 32);

    bmp.size_0=imageHeader[2];        // Set bmp Size
    bmp.size_1=imageHeader[3];
    bmp.size_2=imageHeader[4];
    bmp.size_3=imageHeader[5];

    bmp.offset_0=imageHeader[10];     // Set offset
    bmp.offset_1=imageHeader[11];
    bmp.offset_2=imageHeader[12];
    bmp.offset_3=imageHeader[13];

    bmp.x_size_0=imageHeader[18];     // Set X dimension
    bmp.x_size_1=imageHeader[19];
    bmp.x_size_2=imageHeader[20];
    bmp.x_size_3=imageHeader[21];

    bmp.y_size_0=imageHeader[22];     // Set Y dimension
    bmp.y_size_1=imageHeader[23];
    bmp.y_size_2=imageHeader[24];
    bmp.y_size_3=imageHeader[25];

    bmp.compress_L=imageHeader[26];   // Compress
    bmp.compress_H=imageHeader[27];

    bmp.depth_L=imageHeader[28];      // Color depth
    bmp.depth_H=imageHeader[29];

    x2 = x + bmp.x_size_int0;
    LineLength = 3*bmp.x_size_int0;
    ReadAddr.Long = (imageAdres + bmp.offset_L);
 
    for (iy=0; iy < bmp.y_size_int0; iy++)
    {       
        SPIFlashReadArray(ImageLine, LineLength);
        setXY(x, y+iy, x2, y+iy);

        SEL_DT;
        for (ix=0; ix < LineLength; ix += 3)
        {
            SEL_GP;
            comWritPort(((ImageLine[2+ix]&248)|ImageLine[1+ix]>>5),((ImageLine[1+ix]&28)<<3|ImageLine[ix]>>3));
            DESELECT();
        }
    }
    
}
#endif
