/*******************************************************************************
 * File:   System_RTC.c
 * Author: T McGee
 * Created: July 16, 2012
 ******************************************************************************/
#include "SystemConfig.h"

/*******************************************************************************
 * Function Name:  rtccSet2127
 * Return Value:   void
 * Parameters:     char sec, char min, char hour, char wday, char mday, char mon, char year
 * Description:    BCD used to set time and date of PCF2127 RTCC.
 ******************************************************************************/
void rtccSet2127(char sec, char min, char hour, char wday, char mday, char mon, char year)
{
    char TimeDate[] = {sec, min, hour, mday, wday , mon, year};
    char i;

    modeSPI(SPI_MODE_RTCC, SPI_SPEED_RTCC);

    SEL_RTC;
    comSPI(RTC_WRITE + 0x00);       //Set start address
    comSPI(0b00001000);             //Load parameters
    comSPI(0b00000000);
    comSPI(0b00000000);
    DESELECT();

    SEL_RTC;
    comSPI(RTC_WRITE + 0x0F);       //Set start address
    comSPI(0b00100000);             //Load parameters
    DESELECT();

    SEL_RTC;
    comSPI(RTC_WRITE + 0x03);       //Set start address
    for(i=0; i<7; i++)
    {
        comSPI(TimeDate[i]);        //Load time and date
    }
    DESELECT();
}

/*******************************************************************************
 * Function Name:  rtccLoadTime
 * Return Value:   void
 * Parameters:     void
 * Description:    Transfer time and date from PCF2127 to PIC32 internal RTCC.
 ******************************************************************************/
void rtccLoadTime(void)
{
    rtccTime t0;
    rtccDate d0;
    
    RtccInit();                                 // Init the RTCC. From public library.
    while(RtccGetClkStat()!=RTCC_CLK_ON);       // wait for the SOSC to be actually running and RTCC
                                                // to have its clock source could wait here at most 32ms

    modeSPI(SPI_MODE_RTCC, SPI_SPEED_RTCC);

    t0.rsvd = 0x00;

    SEL_RTC;
    comSPI(RTC_READ + 0x03);                                //Read starting at address 0x00
    t0.sec  = comSPI( 0x00 );
    t0.min  = comSPI( 0x00 );
    t0.hour = comSPI( 0x00 );

    d0.mday = comSPI( 0x00 );
    d0.wday = comSPI( 0x00 );
    d0.mon  = comSPI( 0x00 );
    d0.year = comSPI( 0x00 );
    DESELECT();

    RtccSetTimeDate(t0.l, d0.l);

}

/*******************************************************************************
 * Function Name:  RtccGetTDstring(void)
 * Return Value:   void
 * Parameters:     void
 * Description:    Initialise LCD panel, and Touch Screen.
 ******************************************************************************/

void RtccGetTDstring(char *buffer )
{
	rtccTime  t0,t1;
	rtccDate  d0,d1;
        char apChar = 0x41, temp;

	do
	{
            d0.l=RTCDATE;
            t0.l=RTCTIME;
            t1.l=RTCTIME;
            d1.l=RTCDATE;
	}while((d0.l != d1.l) || (t0.l != t1.l));		// update the user requested data

/***Date***********************************************************************/
        buffer[0] = (((d0.mon & 0b00010000) >> 4) + 48);
        buffer[1] = ((d0.mon & 0b00001111) + 48);
        
        buffer[2] = 0x2F;
        buffer[3] = (((d0.mday & 0b00110000) >> 4) + 48);
        buffer[4] = ((d0.mday & 0b00001111) + 48);

        buffer[5] = 0x2F;
        buffer[6] = 0x32;
        buffer[7] = 0x30;
        buffer[8] = (((d0.year & 0b11110000) >> 4) + 48);
        buffer[9] = ((d0.year & 0b00001111) + 48);
        buffer[10] = 0x20;

/***Time***********************************************************************/
        
        temp = ((t0.hour & 0b00110000) >> 4) * 10;
        temp += (t0.hour & 0b00001111);
        if (temp > 11)
        {
          temp -= 12;
          apChar = 0x50;
        }                   
        if(!temp)
        {
            buffer[11] = 0x31;
            buffer[12] = 0x32;
        }
        else
        {
            buffer[11] = (temp / 10 + 48);
            buffer[12] = (temp % 10 + 48);
        }
        
        buffer[13] = 0x3A;
        buffer[14] = (((t0.min & 0b01110000) >> 4) + 48);
        buffer[15] = ((t0.min & 0b00001111) + 48);
        
        buffer[16] = 0x3A;
        buffer[17] = (((t0.sec & 0b01110000) >> 4) + 48);
        buffer[18] = ((t0.sec & 0b00001111) + 48);

        buffer[19] = 0x20;
        buffer[20] = apChar;
        buffer[21] = 0x4D;
     
        buffer[22] = 0x00;

}


