/******************************************************************************
 * FileName:        MAX32_Buss.c
 * Dependencies:    GenericTypeDefs.h
 *                  string.h
 * Processor:       PIC32
 * Compiler:        CX32
 * Company:         Pacific Coast Motion
******************************************************************************/
#include "SystemConfig.h"

/******************************************************************************
 * Functions
 *****************************************************************************/

/*****************************************************************************
  Function: void Init_Buss(unsigned char Mode,unsigned int sync_rate)
  Summary:  Start parallel and SPI buss
  Conditions: None
  Input: Mode: SPI Mode, sync_rate: Periferal bus divider
  Return: None
  Side Effects: None.
  Description: The Init_Buss function initializes the I/O pins and uart associated
               with the 16bit parallel bus and SPI bus.
  ***************************************************************************/
void Init_Buss( char Mode,long bit_rate)
{
    LATE = 0x00;
    LATD = 0x00;
    GRAPHICS_BUS_TRIS;          //Configure PortD as output for low byte (0:7),
                                //and PortE as output for High byte (8:15)
    LATA = 0b10111111;          //Set Port A for WIFI CS and hold in Reset
    ADDRESS_BUSS_TRIS;          //Set output lines for bus address controller.

    modeSPI(Mode, bit_rate);

}//end Init_Buss

/*****************************************************************************
  Function: void modeSPI(unsigned char Mode,unsigned int sync_rate)
  Summary: Change the mode and speed of the SPI uart
  Conditions: None
  Input: Mode: SPI Mode, bit_rate: SPI buadrate
  Return: None
  Side Effects: None.
  Description: Change the mode and speed of the SPI uart
  ****************************************************************************/
void modeSPI( char Mode, long bit_rate)
{
    SpiOpenFlags oFlags;
    
    SpiChnClose(SPI_CHANNEL4);

         if(Mode == 0) oFlags = SPI_MODE8 | SPI_MASTER | CKP_LOW  | CKE_HIGH | SMP_END;
    else if(Mode == 1) oFlags = SPI_MODE8 | SPI_MASTER | CKP_LOW  | CKE_LOW  | SMP_END;
    else if(Mode == 2) oFlags = SPI_MODE8 | SPI_MASTER | CKP_HIGH | CKE_HIGH | SMP_END;
                  else oFlags = SPI_MODE8 | SPI_MASTER | CKP_HIGH | CKE_LOW  | SMP_END;
 
    // Use SPI4, configure the I/O ports, set spi mode
    SpiChnOpen(SPI_CHANNEL4, oFlags,(unsigned int)(GetSystemClock()/bit_rate));
 
}//end modeSPI

/*****************************************************************************
  Function: unsigned char comSPI (unsigned char data_out)
  Summary: Writes data to the SPI port and read data sent.
  Conditions: SPI and I/O initialized
  Input: data_out: The data to write.
  Return: unsigned char: Recieved data.
  Side Effects: None.
  Description: The comSPI function will write a byte of data from the 
               microcontroller to the device, and read the data sent by the device.
  ****************************************************************************/
char comSPI( char data_out )
{
	SPI4BUF = data_out;          // write to buffer for TX
	while (!SPI4STATbits.SPITBE || !SPI4STATbits.SPIRBF);      // wait for TX complete
	return SPI4BUF;              // read the received values
} // end comSPI

/*****************************************************************************
  Function: void comWritPort(char H_byte, char L_byte)
  Summary: Writes data to the parallel port.
  Conditions: I/O and Address bus initialized
  Input: H_byte,L_byte: The data to write.
  Return: None
  Side Effects: None.
  Description: The comWritPort function will write data from the microcontroller 
               to the device.
  ****************************************************************************/
void comWritPort(char H_byte,char L_byte)
{
    LATE = H_byte;
    LATD = L_byte;
    START_WR;                        // Start of Write cycle
    END_WR;                          // End of Write cycle
}

